#include <cmapl/cmapl_ext.h>
#include <cmapl/cmapl_common_defs.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>
#include <assert.h>
#include <stdlib.h> 
#include <tree.h>

#define __STDC_FORMAT_MACROS
#include <inttypes.h>
#include <objbase.h>

/*
	Description: Get counter names as comma delimited string
	Input: 	data file handle
			object name
	Output: pointer to a string

*/
char* get_counter_names(cmapl_file_handle_t* data_file_handle, char* object, int num_frequencies, int frequencies[])
{
	char return_message[600];
	char return_string[1000];
	return_message[0] = '0';
	return_string[0] = '0';
	int isStarted = 0;

	cmapl_error_t err = CMAPL_SUCCESS;

	// get the header handle for this data file
	cmapl_header_handle_t* header_handle;
	err = cmapl_get_header(data_file_handle, &header_handle);
	if (err != CMAPL_SUCCESS) {
		return strdup("cmapl_get_header failed.");
	}

	// get the header info for this header handle
	cmapl_header_def_t header_info;
	err = cmapl_get_header_info(header_handle, &header_info);
	if (err != CMAPL_SUCCESS) {
		return strdup("cmapl_get_header_info failed.");
	}

	// find the object handle for the desired object
	cmapl_object_handle_t * object_handle;
	err = cmapl_find_object(header_handle, object, &object_handle);
	if (err != CMAPL_SUCCESS) {
		sprintf(return_message, "cmapl_find_object failed with error %d on object %s.", err, object);
		return strdup(return_message);
	}

	// iterate through all counters; finding those that match the desired frequency
	cmapl_counter_handle_t* counter_handle;
	err = cmapl_first_counter(object_handle, &counter_handle);
	if (err != CMAPL_SUCCESS) {
		return strdup("cmapl_first_counter failed.");
	}

	while (counter_handle != NULL && err == CMAPL_SUCCESS) {

		cmapl_counter_def_t ctr_info;
		err = cmapl_get_counter_info(counter_handle, &ctr_info);
		if (err != CMAPL_SUCCESS) {
			return strdup("cmapl_get_counter_info failed.");
		}

		if (!ctr_info.is_key) {
			for (int i = 0; i < ctr_info.num_periods; i++) {
				int frequency_found = 0;
				for (int j = 0; j < num_frequencies; j++) {
					if (frequencies[j] == ctr_info.periods[i]) {
						frequency_found = 1;
						break;
					}
				}
				if (frequency_found) {
					if (!isStarted) {
						strcpy(return_string, ctr_info.name);
						isStarted = 1;
					} else {
						strcat(return_string, ",");
						strcat(return_string, ctr_info.name);
					}
					break;
				}
			}
		}
		err = cmapl_next_counter(object_handle, &counter_handle);
		if (err != CMAPL_SUCCESS) {
			return strdup("cmapl_next_counter failed.");
		}

	}
	return strdup(return_string);
}

/*
	Description: DLL exported function to parse file
	Input: file path, object name, counters, number of counters, frequency
	Output: integer
*/
extern __declspec(dllexport) char* get_counters(char* filepath, char* object, int num_frequencies, int frequencies[]) {

	cmapl_file_handle_t* file;
	const char* meta_path;
	const char* data_path;
	int size;
	char result[10000];

//	char testObjectName[]= "system";

	cmapl_error_t err = cmapl_open_data_file(filepath, "rb", &file, &meta_path, &data_path);
	if (err != CMAPL_SUCCESS){
		strcpy(result,"cmapl_open_data_file failed.");
	} else {
		char* counters = get_counter_names(file, object, num_frequencies, frequencies);
		strcpy(result,counters);
		free(counters);
		if(result == NULL) {
			strcpy(result,"result was null.");
		}
	}
	// close the archive file handle
	err = cmapl_close_data_file(file);

    ULONG ulSize = strlen(result) + sizeof(char);
    char* pszReturn = NULL;
    pszReturn = (char*)CoTaskMemAlloc(ulSize);
    strcpy(pszReturn, result);
    return pszReturn;
}




