#include <cmapl/cmapl_ext.h>
#include <cmapl/cmapl_common_defs.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>
#include <assert.h>
#include <stdlib.h>
#include <tree.h>
char* get_counter_names(cmapl_file_handle_t* data_file_handle, char* object, int num_frequencies, int frequencies[]);

#define __STDC_FORMAT_MACROS
#include <inttypes.h>

/*
	Description: Function to get the query result out of an archive file
	Input: file handler, object name, counters, number of counters, frequemcy, start time, end time
	Output: query handler

*/
cmapl_query_handle_t* get_result_new(cmapl_file_handle_t* file, char* object, char** counters, int num_counters, int frequency, 
								uint64_t start_time, uint64_t end_time)
{
	cmapl_error_t err = CMAPL_SUCCESS;
	cmapl_query_def_t query_def;
	cmapl_set_latest_version(&query_def);

	// assign object info
    // object to query for
	query_def.object_name = object;

	query_def.num_key_cnts = 0;

	// assign counter info
	query_def.num_cnts = num_counters;
	query_def.cnt_names = counters;

	// assign period info
	query_def.period = frequency;                              // only get counter data sampled every 5000ms

	// assign time stamp info
	query_def.start_time = start_time;                      // epoch time in ms to start looking for counter data
	query_def.end_time = end_time;                        // epoch time in ms to stop looking for counter data

	// make query
	cmapl_query_handle_t * query;
	err = cmapl_query(file, &query_def, &query);

	if (err != CMAPL_SUCCESS){
		//printf("CMAPL error: cmapl_function returned error %s (%d)\n", cmapl_error_labels[err], (int) err);
        return NULL;
	}

	return query;
}

/*
	Description: Parse the query result
	Input: query handler
	Output: query handler

*/
int parse_result_new(char* outfilepath, cmapl_query_handle_t* query, char* object, char** csvString)
{
	char currentTimestampAsString[200]; // now each 10x larger than I think they need to be.
	char currentInstanceKeyNames[1280];
	char currentInstanceKeyValues[1280];
	int csvHeaderRowIsWritten = 0;

	// query, of type cmapl_query_handle_t, is retrieved from the previous example
	cmapl_error_t err = CMAPL_SUCCESS;

	// iterate through all the timestamps in the query result that have counter data
	cmapl_timestamp_handle_t* timestamp;
	err = cmapl_first_timestamp(query, &timestamp);

	if (err != CMAPL_SUCCESS){
	    return 0;
	}
	if (err == CMAPL_RECORD_EMPTY){
        return 0;
	}

	FILE *fOut;

	while (timestamp != NULL && err == CMAPL_SUCCESS){

        // dump timestamp
        uint64_t time = 0;
        err = cmapl_get_timestamp(timestamp, &time);
        if (err != CMAPL_SUCCESS){
			break;
        }

        // iterate through all the instances at this timestamp that have data
        cmapl_instance_handle_t* instance = NULL;
        err = cmapl_first_instance(timestamp, &instance);
        if (err != CMAPL_SUCCESS){
			cmapl_next_timestamp(query, &timestamp);
			continue;
        }

		sprintf(currentTimestampAsString,"%"PRId64,time);
		currentInstanceKeyNames[0] = '\0';
		currentInstanceKeyValues[0] = '\0';

        while (instance != NULL && err == CMAPL_SUCCESS){
            // get instance data
            cmapl_instance_data_t instance_data;
            err = cmapl_get_instance_data(instance, &instance_data);
            if (err != CMAPL_SUCCESS){
				continue;
            }

			// save the instance key names and values as comma delimited strings
            uint32_t i;
			strcpy(currentInstanceKeyNames, instance_data.key_cnt_names[0]);
			strcpy(currentInstanceKeyValues, instance_data.key_cnt_values[0].values[0].val_str);
			for (i = 1; i < instance_data.num_key_cnts; i++) {
				strcat(currentInstanceKeyNames, ",");
				strcat(currentInstanceKeyNames, instance_data.key_cnt_names[i]);
				strcat(currentInstanceKeyValues, ",");
				strcat(currentInstanceKeyValues, instance_data.key_cnt_values[i].values[0].val_str);
			}

			// write header row by iterating through all the counter data for first instance

            cmapl_counter_handle_t* ret_cnt;
            cmapl_counter_data_handle_t* cnt_data;

			if (!csvHeaderRowIsWritten) {
				csvHeaderRowIsWritten = 1;

				fOut = fopen(outfilepath, "ab+");
				if (fOut == NULL) {
					perror("Error opening file");
					printf("outputfilepath &s\n", outfilepath);
					printf("Error no: %d\n", errno);
					printf("Error str: %s\n", strerror (errno));
				}

				fputs("TIMESTAMP", fOut);
				fputs(",", fOut);
				fputs(currentInstanceKeyNames, fOut);

	            err = cmapl_first_counter_data(instance, &ret_cnt, &cnt_data);
				if (err != CMAPL_SUCCESS){
					continue;
				}
				while (cnt_data != NULL){
					// get counter info
					cmapl_counter_def_t cnt_info;
    	            err = cmapl_get_counter_info(ret_cnt, &cnt_info);
	                if (err != CMAPL_SUCCESS){
 		               	continue;
					}

	                // get counter data
	                cmapl_counter_data_t cnt_data_value;
	                err = cmapl_get_counter_data_value(cnt_data, &cnt_data_value);
	                if (err != CMAPL_SUCCESS){
						continue;
	                }

					int instanceNameflg = 0;
					char* instnm = "instance_name";
					if(strcmp(cnt_info.name, instnm) == 0) {
						instanceNameflg = 1;
					}

					int vserverNameflg = 0;
					char* vsvrnm = "vserver_name";
					if(strcmp(cnt_info.name, vsvrnm) == 0) {
						vserverNameflg = 1;
					}

	                // dump counter data
	                uint32_t j;
					const char* delim = ",";

					if (!instanceNameflg && !vserverNameflg) {

			                for (j = 0; j < cnt_data_value.num_values; j++){

							// print out counter headers

							fputs(",", fOut);
							fputs(object, fOut);
							fputs(":", fOut);
							fputs(cnt_info.name, fOut);

							if (cnt_info.num_x_items > 1) {
								fputs(":", fOut);
								fputs(cnt_info.x_names[j], fOut);
							}
	 	                }
	                }

	                // get next counter data in this instance
	                err = cmapl_next_counter_data(instance, &ret_cnt, &cnt_data);
	                if (err != CMAPL_SUCCESS){
	                	continue;
	                }
	            }

				// end csv header row
				fputs("\n", fOut);
			}

            err = cmapl_first_counter_data(instance, &ret_cnt, &cnt_data);
            if (err != CMAPL_SUCCESS){
				continue;
            }

			fputs(currentTimestampAsString, fOut);
			fputs(",", fOut);
			fputs(currentInstanceKeyValues, fOut);

            while (cnt_data != NULL){
                // get counter info
                cmapl_counter_def_t cnt_info;
                err = cmapl_get_counter_info(ret_cnt, &cnt_info);
                if (err != CMAPL_SUCCESS){
                	continue;
                }

                // get counter data
                cmapl_counter_data_t cnt_data_value;
                err = cmapl_get_counter_data_value(cnt_data, &cnt_data_value);
                if (err != CMAPL_SUCCESS){
					continue;
                }

				int instanceNameflg = 0;
				char* instnm = "instance_name";
				if(strcmp(cnt_info.name, instnm) == 0) {
					instanceNameflg = 1;
				}

				int vserverNameflg = 0;
				char* vsvrnm = "vserver_name";
				if(strcmp(cnt_info.name, vsvrnm) == 0) {
					vserverNameflg = 1;
				}

                // dump counter data
                uint32_t j;
				const char* delim = ",";
				char work[80];
				int count = 0;
				int hasMore = 0;

				if (!instanceNameflg && !vserverNameflg) {
	                for (j = 0; j < cnt_data_value.num_values; j++){

						fputs(",", fOut);
						// print out counter data value depending on the counter type
	 	    			switch(cnt_data_value.value_type){
							case CMAPL_CNT_4BYTES:
	                            sprintf(work,"%d", cnt_data_value.values[j].val_4B);
	                            fputs(work, fOut);
	                            break;
							case CMAPL_CNT_8BYTES:
	                            sprintf(work, "%"PRIu64, cnt_data_value.values[j].val_8B);
	                            fputs(work, fOut);
	                            break;
							case CMAPL_CNT_STRING:
								fputs((char*)cnt_data_value.values[j].val_str, fOut);
	                            break;
		                }
    	            }
				}

                // get next counter data in this instance
                err = cmapl_next_counter_data(instance, &ret_cnt, &cnt_data);
                if (err != CMAPL_SUCCESS){
                	continue;
                }
            }

			// this is the end of an instance, so end this line of csv data
			fputs("\n", fOut);

            // get next instance at this timestamp
            err = cmapl_next_instance(timestamp, &instance);
        }

        // get the next timestamp in the query result
        err = cmapl_next_timestamp(query, &timestamp);
	}
	fclose(fOut);
	return 1;
}

/*
	Description: free a char**
	Input: char**, size of char* array (number of char* occurrences)
	Output: Array of strings
*/
int free_charss(char** chars_array, int count)
{
	for (int j=0; j < count; j++) {
		free(chars_array[j]);
		chars_array[j] = NULL;
	}

	free(chars_array);
	chars_array = NULL;

	return 0;
}

int count_elements_in_csv(char* csv) {
	if (strlen(csv) == 0) {
		return 0;
	}
	char comma = ',';
	int comma_count = 1;
	for (char *p = csv; *p; p++) {
		if (*p == comma) {
			comma_count++;
		}
	}
	return comma_count + 1;
}
/*
	Description: split a string on delimeters
	Input: string, number of substrings
	Output: Array of strings
*/
char** str_split_new(char* string, int count, char* delim)
{
    char** result;

    result = (char**)malloc(sizeof(char*) * (count + 1));

	char* token = NULL;

	int indx = 0;

	if (string != NULL) {

	  token = strtok(string,delim);

	  while (token)
	  {
		if(indx >= count) {
			break;
		}
		result[indx] = strdup(token);
		token = strtok(NULL,delim);
		indx++;
	  }

	  result[indx] = NULL;
	}

    return result;
}

/*
	Description: DLL exported function to parse file
	Input: file path, object name, counters, number of counters, frequency
	Output: integer
*/
extern __declspec(dllexport) int get_csv(char* in_file_path, char* out_file_path, char* object, int num_objects, char* counters_str, char* num_counters, int frequency ) {
	cmapl_file_handle_t* file;
	const char* meta_path;
	const char* data_path;
	char** object_arr = str_split_new(object, num_objects, ",");
	char** object_counter_arr = str_split_new(counters_str, num_objects, "|");
	char** num_counters_arr = str_split_new(num_counters, num_objects, ",");
	int size;
	char* resultString;
	int returnCode = 0;

	cmapl_error_t err = cmapl_open_data_file(in_file_path, "rb", &file, &meta_path, &data_path);
	if (err != CMAPL_SUCCESS){
	} else {
		returnCode = 0;
		uint64_t old_time;
		cmapl_get_oldest_record_time(file, &old_time);

		uint64_t new_time;
		cmapl_get_newest_record_time(file, &new_time);

		for(int i = 0; i< num_objects;i++) {
			char *pv;
			int num_counter = strtol(num_counters_arr[i],&pv,10);
			char** counters_arr = str_split_new(object_counter_arr[i],num_counter, ",");

			cmapl_query_handle_t* result = get_result_new(file, object_arr[i], counters_arr, num_counter, frequency, old_time, new_time);

			free_charss(counters_arr, num_counter);
			counters_arr = NULL;

			if(result == NULL) {
				returnCode = 1;
				break;
			}

			int ret = parse_result_new(out_file_path, result, object_arr[i], &resultString);
			if (ret == 0){
				returnCode = 1;
				break;
			}
		}
	}

	// close the archive file handle
	err = cmapl_close_data_file(file);

	free_charss(object_arr, num_objects);
	object_arr = NULL;

	free_charss(object_counter_arr, num_objects);
	object_counter_arr = NULL;

	free_charss(num_counters_arr, num_objects);
	num_counters_arr = NULL;

	return returnCode;

}

/*
	Name: get_counters_by_object

	param[in]	in_file_path		The input path of the ccma files (excluding the trailing ".gz" and ".meta")
	param[in]	out_file_path		The path to be written with the csv collected
	param[in]	object				The name of the object for which counters are to collected
	param[in]	num_frequencies		The size of the frequencies array
	param[in]	frequencies			The counters to be collected, i.e., the counters to select when parsing the ccma.meta file.
	param[in]	instance_frequency	The instance frequency, i.e., defines the instances to select when parsing the ccma.gz file.
	This is a DLL exported function to parse a ccma file and writing the counter data to a csv file specified by out_file_path.
*/
extern __declspec(dllexport) int get_counters_by_object(char* in_file_path, char* out_file_path, char* object, int num_frequencies, int frequencies[], int instance_frequency) {
	cmapl_file_handle_t* file;
	const char* meta_path;
	const char* data_path;
	int size;
	char* resultString;
	int returnCode = 0;

	cmapl_error_t err = cmapl_open_data_file(in_file_path, "rb", &file, &meta_path, &data_path);
	if (err != CMAPL_SUCCESS){
		returnCode = 1;
	} else {

		char* counter_name_csv = get_counter_names(file, object, num_frequencies, frequencies);
		int num_counter = count_elements_in_csv(counter_name_csv);

		char** counters_arr = str_split_new(counter_name_csv, num_counter, ",");

		uint64_t old_time;
		cmapl_get_oldest_record_time(file, &old_time);

		uint64_t new_time;
		cmapl_get_newest_record_time(file, &new_time);

		char *pv;

		cmapl_query_handle_t* result = get_result_new(file, object, counters_arr, num_counter, instance_frequency, old_time, new_time);

		free_charss(counters_arr, num_counter);
		counters_arr = NULL;

		if(result == NULL) {
			returnCode = 1;
		}

		int ret = parse_result_new(out_file_path, result, object, &resultString);
		if (ret == 0){
			returnCode = 1;
		}
	}

	// close the archive file handle
	err = cmapl_close_data_file(file);

	return returnCode;

}
