#include <cmapl/cmapl_ext.h>
#include <cmapl/cmapl_common_defs.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>
#include <assert.h>
#include <stdlib.h> 
#include <tree.h>

#define __STDC_FORMAT_MACROS
#include <inttypes.h>

xmlDocPtr doc = NULL;
int initialize_doc = 0;

/*
	Description: Function to get the query result out of an archive file
	Input: file handler, object name, counters, number of counters, frequemcy, start time, end time
	Output: query handler 

*/
cmapl_query_handle_t* get_result(cmapl_file_handle_t* file, char* object, char** counters, int num_counters, int frequency, 
								uint64_t start_time, uint64_t end_time)
{
	cmapl_error_t err = CMAPL_SUCCESS;
	cmapl_query_def_t query_def;
	cmapl_set_latest_version(&query_def);

	// assign object info
    // object to query for
	query_def.object_name = object;

	query_def.num_key_cnts = 0;

	// assign counter info
	query_def.num_cnts = num_counters;	
	query_def.cnt_names = counters;

	// assign period info
	query_def.period = frequency;                              // only get counter data sampled every 5000ms

	// assign time stamp info
	query_def.start_time = start_time;                      // epoch time in ms to start looking for counter data
	query_def.end_time = end_time;                        // epoch time in ms to stop looking for counter data

	// make query
	cmapl_query_handle_t * query;
	err = cmapl_query(file, &query_def, &query);

	if (err != CMAPL_SUCCESS){
		//printf("CMAPL error: cmapl_function returned error %s (%d)\n", cmapl_error_labels[err], (int) err);
        return NULL;
	}

	//free(&query_def);
	// return result
	return query;

}

/*
	Description: Counts the number of digits in an integer
*/
int count_digit(int number) {
	int count = 0;
	for(;number!=0;number = number/10)
		count++;
	return count;
}

/*
	Description: Parse the query result
	Input: query handler
	Output: query handler 

*/
int parse_result(cmapl_query_handle_t* query, char* object)
{
	// query, of type cmapl_query_handle_t, is retrieved from the previous example
	cmapl_error_t err = CMAPL_SUCCESS;

	// iterate through all the timestamps in the query result that have counter data
	cmapl_timestamp_handle_t* timestamp;
	err = cmapl_first_timestamp(query, &timestamp);
	if (err != CMAPL_SUCCESS){
	    return 0;
	}
	if (err == CMAPL_RECORD_EMPTY){
        return 0;
	} 

	xmlNodePtr tree, cnttree, timetree, insttree;

	while (timestamp != NULL && err == CMAPL_SUCCESS){
        // dump timestamp
        uint64_t time = 0;
        err = cmapl_get_timestamp(timestamp, &time);
        if (err != CMAPL_SUCCESS){
			break;
        }

		if(initialize_doc == 0) {
			doc = xmlNewDoc("1.0");
			doc->children = xmlNewDocNode(doc, NULL, "asup", NULL);
			initialize_doc = 1;
		}
		char object_str[50];
		sprintf(object_str,"ob-%s",object);
		tree = xmlNewChild(doc->children, NULL, object_str, NULL);
		

		char timestamp_str[30];
		sprintf(timestamp_str,"t%"PRId64,time);
		timetree = xmlNewChild(tree, NULL, timestamp_str, NULL);

        // iterate through all the instances at this timestamp that have data
        cmapl_instance_handle_t* instance = NULL;
        err = cmapl_first_instance(timestamp, &instance);
        if (err != CMAPL_SUCCESS){
			cmapl_next_timestamp(query, &timestamp);
			continue;
        }
        while (instance != NULL && err == CMAPL_SUCCESS){
            // get instance data
            cmapl_instance_data_t instance_data;
            err = cmapl_get_instance_data(instance, &instance_data);
            if (err != CMAPL_SUCCESS){
				continue;
            }
			
             // iterate through all the counter data in this instance
            cmapl_counter_handle_t* ret_cnt;
            cmapl_counter_data_handle_t* cnt_data;
            err = cmapl_first_counter_data(instance, &ret_cnt, &cnt_data);
            if (err != CMAPL_SUCCESS){
				continue;
            }
            while (cnt_data != NULL){
                // get counter info
                cmapl_counter_def_t cnt_info;
                err = cmapl_get_counter_info(ret_cnt, &cnt_info);
                if (err != CMAPL_SUCCESS){
                	continue;
                }

                // get counter data
                cmapl_counter_data_t cnt_data_value;
                err = cmapl_get_counter_data_value(cnt_data, &cnt_data_value);
                if (err != CMAPL_SUCCESS){
					continue;
                }

				int instflg = 0;
				char* instnm = "instance_name";
				if(strcmp(cnt_info.name, instnm) == 0) {
					instflg = 1;
				} 
				
                // dump counter data
                uint32_t j;
				char* temp_value;
				char* final_value;
				char* value;
				const char* delim = ",";
				int val = 0;
				int count = 0;
				int hasMore = 0;
				
                for (j = 0; j < cnt_data_value.num_values; j++){
						
	               // print out counter data value depending on the counter type
	               switch(cnt_data_value.value_type){
	                    case CMAPL_CNT_4BYTES:
								val = cnt_data_value.values[j].val_4B;
								count = count_digit(val);
								value = (char*)(malloc((count + 1) * 4));
	                            sprintf(value,"%d", cnt_data_value.values[j].val_4B);
	                            break;
	                    case CMAPL_CNT_8BYTES:
								val = cnt_data_value.values[j].val_8B;
								count = count_digit(val);
								value = (char*)(malloc((count + 1) * 8));
	                            sprintf(value, "%"PRIu64, cnt_data_value.values[j].val_8B);
	                            break;
	                    case CMAPL_CNT_STRING:
								value = strdup((char*)cnt_data_value.values[j].val_str);
	                            break;
	                }
					if(hasMore==1) {
						
						final_value = (char*)malloc((strlen(value)+strlen(temp_value)+strlen(delim))*sizeof(char));						
						strcpy(final_value,temp_value);
						strcat(final_value,delim);
						strcat(final_value,value);
					} else {
						final_value = strdup(value);
					}

	                if (j != (cnt_data_value.num_values - 1)){
	                 	temp_value = strdup(final_value);
						//free(final_value);
						hasMore = 1;
	                }
					free(value);
                }
                
				if(instflg == 1) {
					insttree = xmlNewChild(timetree, NULL, final_value, NULL);
				} else {
					cnttree = xmlNewChild(insttree, NULL, cnt_info.name, final_value);
				}	

				//free(temp_value);
				free(final_value);	

                // get next counter data in this instance
                err = cmapl_next_counter_data(instance, &ret_cnt, &cnt_data);

                if (err != CMAPL_SUCCESS){
                	continue;
                }
            }

            // get next instance at this timestamp
            err = cmapl_next_instance(timestamp, &instance);
        }

        // get the next timestamp in the query result
        err = cmapl_next_timestamp(query, &timestamp);
	}

	return 1;
}

/*
	Description: split a string on delimeters
	Input: string, number of substrings
	Output: Array of strings
*/
char** str_split(char* string, int count, char* delim)
{
    char** result  ;
	
    result = (char**)malloc(sizeof(char*) * (count + 1));

	char* token = NULL;

	int indx = 0;
	
	if (string != NULL) {

	  token = strtok(string,delim);

	  while (token)
	  {
		if(indx >= count) {
			break;
		}
		result[indx] = strdup(token);
		token = strtok(NULL,delim);
		indx++;
	  }

	  result[indx] = NULL;
	}
	
    return result;
}

/*
	Description: DLL exported function to parse file
	Input: file path, object name, counters, number of counters, frequency
	Output: integer
*/
extern __declspec(dllexport) char* parse_file(char* file_path, char* object, int num_objects, char* counters_str, char* num_counters, int frequency ) {
	cmapl_file_handle_t* file;
	const char* meta_path;
	const char* data_path;
	char** object_arr = str_split(object, num_objects, ",");
	char** object_counter_arr = str_split(counters_str, num_objects, "|");
	char** num_counters_arr = str_split(num_counters, num_objects, ",");
	xmlChar* s;
	int size;
	initialize_doc = 0;

	if (doc != NULL) {
//		free(doc);
		doc = NULL;
	}

	cmapl_error_t err = cmapl_open_data_file(file_path, "rb", &file, &meta_path, &data_path);
	if (err != CMAPL_SUCCESS){
			doc = xmlNewDoc("1.0");
			doc->children = xmlNewDocNode(doc, NULL, "asup", NULL);
			char buf[160];
			sprintf(buf, "Error %d", err);
			xmlNewChild(doc->children, NULL, "error", buf);
	} else {
		uint64_t old_time;
		cmapl_get_oldest_record_time(file, &old_time);

		uint64_t new_time;
		cmapl_get_newest_record_time(file, &new_time);		
		
		for(int i = 0; i< num_objects;i++) {
			char *pv;
			int num_counter = strtol(num_counters_arr[i],&pv,10);
			char** counters_arr = str_split(object_counter_arr[i],num_counter, ",");
			
			cmapl_query_handle_t* result = get_result(file, object_arr[i], counters_arr, num_counter, frequency, old_time, new_time);
			//free(pv);

			counters_arr = NULL;

			if(result == NULL) {
				continue;
			}	

			int ret = parse_result(result, object_arr[i]);
			if (ret == 0){
				printf("Could not parse the results for query of object %s\n", object_arr[i]);
			}
		}
	}

	// close the archive file handle
	err = cmapl_close_data_file(file);


	char* out = NULL;

	if (doc != NULL) {
		xmlDocDumpMemoryEnc(doc, &s, &size, "UTF-8");

		out = (char*)s;
	}

	return out;

}




