/**
 * @file cmapl_mem_defs.h
 *
 * The CMAPL memory map definitions file. It contains the macros and function
 * declarations necessary to capture filesystem calls from CMAPL API and
 * redirect these to use memory mapped implementations instead.
 *
 */

#ifndef _CMAPL_MEM_DEFS_H_
#define _CMAPL_MEM_DEFS_H_

#define Byte no_conflict_Byte
#include <zlib.h>
#undef Byte

#define CMAPL_GZOPEN (CMAPL_GZFILE)cmapl_mem_gzopen
#define CMAPL_GZSEEK cmapl_mem_gzseek
#define CMAPL_GZREAD cmapl_mem_gzread
#define CMAPL_GZCLOSE cmapl_mem_gzclose

// custom memory mapped file handle that will be casted
// and passed back to the user
typedef struct {
	int fd;
	const char* file_path;
	uint64_t file_size;
	uint64_t file_offset;
	uint64_t block_start_offset;
	uint64_t block_end_offset;
	uint64_t block_size;
	char* file_addr;
	char* temp_dir;
} cmapl_mmap_file;

extern gzFile
cmapl_mem_gzopen(
			const char* data_file_path,
			const char* access_type);

extern long int
cmapl_mem_gzseek(
			gzFile file,
			long offset,
			int origin);

extern int
cmapl_mem_gzread(
			void * buffer,
			unsigned int size,
			gzFile file);

extern int
cmapl_mem_gzclose(
			gzFile file);

#endif /* _CMAPL_MEM_DEFS_H_  */
