#include "cmapl_common_defs.h"
#include "cmapl_int_defs.h"
#include "cmapl_write.h"
#include "cmapl_write_int.h"

// Define CMAPL_ERROR_FORMAT to generate an array of strings.
#define CMAPL_ERROR_FORMAT(i) #i
// array of error strings
const char * cmapl_error_labels[CMAPL_ERROR_CODES_MAX_INDEX] =
	{ "CMAPL_SUCCESS", CMAPL_ERROR_CODES()};
#undef CMAPL_ERROR_FORMAT

#ifdef _KERNEL
/* Defines the CMAPL Memory pool when running in Ontap/Dblade */
MALLOC_DEFINE(M_CMAPL, "cmapl", "Clustered CM Archiver Parsing Library Memory");
#endif

// array of all possible periods
const uint64_t _cmapl_period_map[] = {	1000,           // 1 second
					5*1000,         // 5 seconds
					10*1000,        // 10 seconds
					30*1000,        // 30 seconds
					60*1000,        // 1 minute
					5*60*1000,      // 5 minutes
					10*60*1000,     // 10 minutes
					30*60*1000,     // 30 minutes
					60*60*1000,     // 1 hour
					6*60*60*1000,   // 6 hours
					12*60*60*1000,  // 12 hours
					24*60*60*1000,  // 1 day
					7*24*60*60*1000,// 1 week
					CMAPL_CONFIG_PERIOD}; // Config period

static cmapl_error_t
cmapl_add_counter_to_list_v6_0(cmapl_counter_handle_t** counter_list, cmapl_counter_handle_t* counter, cmapl_cnt_group_t type, uint64_t period)
{
	uint32_t period_index = 0;
	if (type == CMAPL_CNT_PERIOD){
		cmapl_error_t err = cmapl_get_period_index(period, &period_index);
		if (err != CMAPL_SUCCESS){
			return err;
		}
	}

        cmapl_counter_handle_t* prev_counter = NULL;
        cmapl_counter_handle_t* current_counter = *counter_list;
        while(current_counter != NULL){
                if (current_counter->e6_0.cnt_id > counter->e6_0.cnt_id){
                        break;
                }
                prev_counter = current_counter;
                if (type == CMAPL_CNT_ID){
                        current_counter = current_counter->i6_0.next_cnt_by_id;
                }else if (type == CMAPL_CNT_KEY){
                        current_counter = current_counter->i6_0.next_cnt_by_key;
                }else if (type == CMAPL_CNT_PERIOD){
                        current_counter = current_counter->i6_0.next_cnt_by_period[period_index];
                }
        }
        if(prev_counter == NULL){
                if (type == CMAPL_CNT_ID){
                        counter->i6_0.next_cnt_by_id = *counter_list;
                }else if (type == CMAPL_CNT_KEY){
                        counter->i6_0.next_cnt_by_key = *counter_list;
                }else if (type == CMAPL_CNT_PERIOD){
                        counter->i6_0.next_cnt_by_period[period_index] = *counter_list;
                }
                *counter_list = counter;
        }else{
                if (type == CMAPL_CNT_ID){
                        prev_counter->i6_0.next_cnt_by_id = counter;
                        counter->i6_0.next_cnt_by_id = current_counter;
                }else if (type == CMAPL_CNT_KEY){
                        prev_counter->i6_0.next_cnt_by_key = counter;
                        counter->i6_0.next_cnt_by_key = current_counter;
                }else if (type == CMAPL_CNT_PERIOD){
                        prev_counter->i6_0.next_cnt_by_period[period_index] = counter;
                        counter->i6_0.next_cnt_by_period[period_index] = current_counter;
                }
        }
        return CMAPL_SUCCESS;
}

cmapl_error_t
cmapl_add_counter_to_list(cmapl_counter_handle_t** counter_list, cmapl_counter_handle_t* counter, cmapl_cnt_group_t type, uint64_t period)
{
	// invalid argument
	if(counter == NULL || type > CMAPL_CNT_PERIOD){
		return CMAPL_INVALID_ARGUMENTS;
	}
	// versions don't match
	if(*counter_list != NULL && !cmapl_is_version(*counter_list, counter->major_v, counter->minor_v)){
		return CMAPL_VERSION_MISMATCH;
	}
	if(cmapl_is_version(counter, 6, 0)){
		cmapl_error_t err = cmapl_add_counter_to_list_v6_0(counter_list, counter, type, period);
		if (err != CMAPL_SUCCESS){
			return err;
		}
	}

	return CMAPL_SUCCESS;
}

static cmapl_error_t
cmapl_add_object_to_list_v6_0(cmapl_object_handle_t** object_list, cmapl_object_handle_t* object)
{
        cmapl_object_handle_t* prev_object = NULL;
        cmapl_object_handle_t* current_object = *object_list;
        while(current_object != NULL){
                if (current_object->e6_0.obj_id > object->e6_0.obj_id){
                        break;
                }
                prev_object = current_object;
                current_object = current_object->i6_0.next_object;
        }
        if(prev_object == NULL){
                object->i6_0.next_object = *object_list;
                *object_list = object;
        }else{
                prev_object->i6_0.next_object = object;
                object->i6_0.next_object = current_object;
        }
        return CMAPL_SUCCESS;
}

cmapl_error_t
cmapl_add_object_to_list(cmapl_object_handle_t** object_list, cmapl_object_handle_t* object)
{
        // invalid argument
        if(object == NULL){
                return CMAPL_NULL_POINTER;
        }
        // versions don't match
        if(*object_list != NULL && !cmapl_is_version(*object_list, object->major_v, object->minor_v)){
                return CMAPL_VERSION_MISMATCH;
        }
        if(cmapl_is_version(object, 6, 0)){
		cmapl_error_t err = cmapl_add_object_to_list_v6_0(object_list, object);
		if (err != CMAPL_SUCCESS){
			return err;
		}
        }

        return CMAPL_SUCCESS;
}

cmapl_error_t
cmapl_copy_attr_def_v6_0(cmapl_attribute_def_t_v6_0 * src, cmapl_attribute_def_t_v6_0 * dst)
{
	// check if arguments are NULL
	if (src == NULL || dst == NULL){
		return CMAPL_NULL_POINTER;
	}

	// check if version is valid
	if (!cmapl_is_version(src, 6, 0)){
		return CMAPL_INVALID_VERSION;
	}

	// assert dst pointers are NULL
	CMAPL_ASSERT(dst->name == NULL);
	CMAPL_ASSERT(dst->desc == NULL);

	dst->major_v = 6;
        dst->minor_v = 0;
        dst->value = src->value;
        dst->name = CMAPL_STRDUP(src->name);
        if (dst->name == NULL){
                return CMAPL_NULL_POINTER;
        }
        dst->desc = CMAPL_STRDUP(src->desc);
        if (dst->desc == NULL){
                return CMAPL_NULL_POINTER;
        }
	return CMAPL_SUCCESS;
}

cmapl_error_t
cmapl_copy_counter_def_v6_0(cmapl_counter_def_t_v6_0 * src, cmapl_counter_def_t_v6_0 * dst)
{
	// check if arguments are NULL
	if (src == NULL || dst == NULL){
                return CMAPL_NULL_POINTER;
        }

	// check if version is valid
        if (!cmapl_is_version(src, 6, 0)){
                return CMAPL_INVALID_VERSION;
        }

	// assert dst pointers are NULL
        CMAPL_ASSERT(dst->name == NULL);
        CMAPL_ASSERT(dst->desc == NULL);
	CMAPL_ASSERT(dst->priv == NULL);
        CMAPL_ASSERT(dst->x_names == NULL);
	CMAPL_ASSERT(dst->y_names == NULL);

	dst->major_v = 6;
	dst->minor_v = 0;
	dst->ctype = src->ctype;
        dst->size = src->size;
        dst->is_key = src->is_key;
        dst->period = src->period;
        dst->cnt_id = src->cnt_id;
        dst->base_id = src->base_id;
        dst->num_x_items = src->num_x_items;
        dst->num_y_items = src->num_y_items;
        dst->name = CMAPL_STRDUP(src->name);
        if (dst->name == NULL){
                return CMAPL_NULL_POINTER;
        }
        dst->priv = CMAPL_STRDUP(src->priv);
        if (dst->priv == NULL){
                return CMAPL_NULL_POINTER;
        }
        dst->desc = CMAPL_STRDUP(src->desc);
        if (dst->desc == NULL){
                return CMAPL_NULL_POINTER;
        }

        // copy over the x label names
        if (src->num_x_items != 0){
                dst->x_names = (const char**)CMAPL_ZALLOC(src->num_x_items * sizeof(const char*));
                if (dst->x_names == NULL){
                        return CMAPL_NULL_POINTER;
                }
		uint32_t i;
                for(i = 0; i < src->num_x_items; i++){
                        dst->x_names[i] = CMAPL_STRDUP(src->x_names[i]);
                        if (dst->x_names[i] == NULL){
                                return CMAPL_NULL_POINTER;
                        }
                }
        } else{
                dst->x_names = NULL;
        }

	// copy over the y label names
        if (src->num_y_items != 0){
                dst->y_names = (const char**)CMAPL_ZALLOC(src->num_y_items * sizeof(const char*));
                if (dst->y_names == NULL){
                        return CMAPL_NULL_POINTER;
                }
		uint32_t i;
                for(i = 0; i < src->num_y_items; i++){
                        dst->y_names[i] = CMAPL_STRDUP(src->y_names[i]);
                        if (dst->y_names[i] == NULL){
                                return CMAPL_NULL_POINTER;
                        }
                }
        } else{
                dst->y_names = NULL;
        }

	return CMAPL_SUCCESS;
}

cmapl_error_t
cmapl_copy_directory_def_v6_0(cmapl_directory_def_t_v6_0 * src, cmapl_directory_def_t_v6_0 * dst)
{
	// check if arguments are NULL
	if (src == NULL || dst == NULL){
                return CMAPL_NULL_POINTER;
        }

	// check if version is valid
        if (!cmapl_is_version(src, 6, 0)){
                return CMAPL_INVALID_VERSION;
        }

	// assert dst pointers are NULL
        CMAPL_ASSERT(dst->directory_path == NULL);

	dst->major_v = 6;
        dst->minor_v = 0;
	dst->start_time = src->start_time;
        dst->end_time = src->end_time;
        dst->period = src->period;
        dst->directory_path = CMAPL_STRDUP(src->directory_path);
        if (dst->directory_path == NULL){
                return CMAPL_NULL_POINTER;
        }

	return CMAPL_SUCCESS;
}

cmapl_error_t
cmapl_copy_header_def_v6_0(cmapl_header_def_t_v6_0 * src, cmapl_header_def_t_v6_0 * dst)
{
	// check if arguments are NULL
	if (src == NULL || dst == NULL){
                return CMAPL_NULL_POINTER;
        }

	// check if version is valid
        if (!cmapl_is_version(src, 6, 0)){
                return CMAPL_INVALID_VERSION;
        }

	// assert dst pointers are NULL
        CMAPL_ASSERT(dst->preset_version == NULL);

	dst->major_v = 6;
        dst->minor_v = 0;
	dst->generation_id = src->generation_id;
        dst->preset_version = CMAPL_STRDUP(src->preset_version);
        if (dst->preset_version == NULL){
                return CMAPL_NULL_POINTER;
        }

	return CMAPL_SUCCESS;
}

cmapl_error_t
cmapl_copy_object_def_v6_0(cmapl_object_def_t_v6_0 * src, cmapl_object_def_t_v6_0 * dst)
{
	// check if arguments are NULL
	if (src == NULL || dst == NULL){
                return CMAPL_NULL_POINTER;
        }

	// check if version is valid
        if (!cmapl_is_version(src, 6, 0)){
                return CMAPL_INVALID_VERSION;
        }

	// assert dst pointers are NULL
        CMAPL_ASSERT(dst->name == NULL);

	dst->major_v = 6;
	dst->minor_v = 0;
	dst->obj_id = src->obj_id;
        dst->name = CMAPL_STRDUP(src->name);
        if (dst->name == NULL){
                return CMAPL_NULL_POINTER;
        }

	return CMAPL_SUCCESS;
}

cmapl_error_t
cmapl_copy_query_def_v6_0(cmapl_query_def_t_v6_0 * src, cmapl_query_def_t_v6_0 * dst)
{
	// check if arguments are NULL
	if (src == NULL || dst == NULL){
                return CMAPL_NULL_POINTER;
        }

	// check if version is valid
        if (!cmapl_is_version(src, 6, 0)){
                return CMAPL_INVALID_VERSION;
        }

	// assert dst pointers are NULL
        CMAPL_ASSERT(dst->object_name == NULL);
        CMAPL_ASSERT(dst->key_cnt_names == NULL);
        CMAPL_ASSERT(dst->key_cnt_values == NULL);
        CMAPL_ASSERT(dst->cnt_names == NULL);

	dst->major_v = 6;
        dst->minor_v = 0;
	dst->num_key_cnts = src->num_key_cnts;
        dst->num_cnts = src->num_cnts;
        dst->start_time = src->start_time;
        dst->end_time = src->end_time;
        dst->period = src->period;
        dst->object_name = CMAPL_STRDUP(src->object_name);
        if (dst->object_name == NULL){
                return CMAPL_NULL_POINTER;
        }

	// copy over key counter names
        uint32_t num_key_cnts = src->num_key_cnts;
        dst->key_cnt_names = (const char**)CMAPL_ZALLOC(num_key_cnts * sizeof(const char*));
        if (dst->key_cnt_names == NULL){
                return CMAPL_NULL_POINTER;
        }
        uint32_t i;
        for(i = 0; i < num_key_cnts; i++){
                dst->key_cnt_names[i] = CMAPL_STRDUP(src->key_cnt_names[i]);
                if (dst->key_cnt_names[i] == NULL){
                        return CMAPL_NULL_POINTER;
                }
        }

	// copy over key counter values
        dst->key_cnt_values = (cmapl_counter_data_t_v6_0*)CMAPL_ZALLOC(num_key_cnts * sizeof(cmapl_counter_data_t_v6_0));
        if (dst->key_cnt_values == NULL){
                return CMAPL_NULL_POINTER;
        }
        for(i = 0; i < num_key_cnts; i++){
                cmapl_counter_data_t_v6_0 key_cnt_value = src->key_cnt_values[i];
                uint32_t num_values = key_cnt_value.num_values;
		dst->key_cnt_values[i].num_values = num_values;
		dst->key_cnt_values[i].value_type = key_cnt_value.value_type;
                dst->key_cnt_values[i].values = (cmapl_cnt_value_t*)CMAPL_ZALLOC(num_values * sizeof(cmapl_cnt_value_t));
                if (key_cnt_value.values == NULL){
			return CMAPL_NULL_POINTER;
                }

                uint32_t j;
                for (j = 0; j < num_values; j++){
                        dst->key_cnt_values[i].values[j] = key_cnt_value.values[j];
                        if (dst->key_cnt_values[j].value_type == CMAPL_CNT_STRING){
                                dst->key_cnt_values[i].values[j].val_str =
                                        CMAPL_STRDUP(key_cnt_value.values[j].val_str);
                                if (dst->key_cnt_values[i].values[j].val_str == NULL){
					return CMAPL_NULL_POINTER;
                                }
                        }
                }
        }

	return CMAPL_SUCCESS;
}

cmapl_error_t
cmapl_copy_system_def_v6_0(cmapl_system_def_t_v6_0 * src, cmapl_system_def_t_v6_0 * dst)
{
	// check if arguments are NULL
	if (src == NULL || dst == NULL){
                return CMAPL_NULL_POINTER;
        }

	// check if version is valid
        if (!cmapl_is_version(src, 6, 0)){
                return CMAPL_INVALID_VERSION;
        }

	// assert dst pointers are NULL
        CMAPL_ASSERT(dst->ontap_version == NULL);
        CMAPL_ASSERT(dst->node_uuid == NULL);
        CMAPL_ASSERT(dst->hostname == NULL);
        CMAPL_ASSERT(dst->model == NULL);
	CMAPL_ASSERT(dst->serial_number == NULL);
        CMAPL_ASSERT(dst->system_id == NULL);

	dst->major_v = 6;
        dst->minor_v = 0;
	dst->base_time = src->base_time;
        dst->ontap_version = CMAPL_STRDUP(src->ontap_version);
        if (dst->ontap_version == NULL){
                return CMAPL_NULL_POINTER;
        }
        dst->node_uuid = CMAPL_STRDUP(src->node_uuid);
        if (dst->node_uuid == NULL){
		return CMAPL_NULL_POINTER;
        }
        dst->hostname = CMAPL_STRDUP(src->hostname);
        if (dst->hostname == NULL){
		return CMAPL_NULL_POINTER;
        }
        dst->model = CMAPL_STRDUP(src->model);
        if (dst->model == NULL){
		return CMAPL_NULL_POINTER;
        }
        dst->serial_number = CMAPL_STRDUP(src->serial_number);
        if (dst->serial_number == NULL){
		return CMAPL_NULL_POINTER;
        }
        dst->system_id = CMAPL_STRDUP(src->system_id);
        if (dst->system_id == NULL){
		return CMAPL_NULL_POINTER;
        }

	return CMAPL_SUCCESS;
}

cmapl_error_t
cmapl_xml_encode(uint32_t num_strs, const char** srcs, char** dsts, bool* encoded_flags)
{
	uint32_t encoded_lens[num_strs];

	// flag to indicate if at least on string needs encoding
	bool needs_encoding = false;

	// check which src strings need encoding, and how long
	// each encoded string should be.
	uint32_t i;
	for (i = 0; i < num_strs; i++){
		encoded_lens[i] = 0;
		uint32_t len = (uint32_t)CMAPL_STRLEN(srcs[i]);
		uint32_t j;
		for (j = 0; j < len; j++){
			char a = srcs[i][j];
			switch (a) {
				case '<':
					encoded_lens[i] += 4;
					break;
				case '>':
					encoded_lens[i] += 4;
                                        break;
				case '\'':
					encoded_lens[i] += 6;
                                        break;
				case '\"':
					encoded_lens[i] += 6;
                                        break;
				case '&':
					encoded_lens[i] += 5;
                                        break;
				default:
					encoded_lens[i]++;
                                        break;
			}
		}
		if (len < encoded_lens[i]){
			encoded_flags[i] = true;
			needs_encoding = true;
		} else {
			encoded_flags[i] = false;
		}
	}

	if (!needs_encoding){
		for (i = 0; i < num_strs; i++){
			dsts[i] = (char*)srcs[i];
		}
		return CMAPL_SUCCESS;
	}

	// allocate memory for each encoded string, and construct
	// the encoded string.
        for (i = 0; i < num_strs; i++){
		if (!encoded_flags[i]){
			dsts[i] = (char*)srcs[i];
			continue;
		}
		dsts[i] = (char*)CMAPL_ZALLOC((encoded_lens[i] + 1) * sizeof(char));
		if (dsts[i] == NULL){
			// unflag the strings that needed encoding that haven't been allocated yet
			uint32_t j;
			for (j = i; j < num_strs; j++){
				encoded_flags[j] = false;
			}
			return CMAPL_NULL_POINTER;
		}
		uint32_t p = 0;
		uint32_t len = (uint32_t)CMAPL_STRLEN(srcs[i]);
                uint32_t j;
                for (j = 0; j < len; j++){
			char a = srcs[i][j];
			switch (a) {
				case '<':
                                        dsts[i][p++] = '&';
                                        dsts[i][p++] = 'l';
                                        dsts[i][p++] = 't';
                                        dsts[i][p++] = ';';
					break;
				case '>':
					dsts[i][p++] = '&';
					dsts[i][p++] = 'g';
					dsts[i][p++] = 't';
					dsts[i][p++] = ';';
					break;
				case '\'':
					dsts[i][p++] = '&';
					dsts[i][p++] = 'a';
					dsts[i][p++] = 'p';
					dsts[i][p++] = 'o';
					dsts[i][p++] = 's';
					dsts[i][p++] = ';';
					break;
				case '\"':
					dsts[i][p++] = '&';
					dsts[i][p++] = 'q';
					dsts[i][p++] = 'u';
					dsts[i][p++] = 'o';
					dsts[i][p++] = 't';
					dsts[i][p++] = ';';
					break;
				case '&':
					dsts[i][p++] = '&';
					dsts[i][p++] = 'a';
					dsts[i][p++] = 'm';
					dsts[i][p++] = 'p';
					dsts[i][p++] = ';';
					break;
				default:
					dsts[i][p++] = a;
			}
		}
		dsts[i][p] = '\0';
	}
	return CMAPL_SUCCESS;
}

void
cmapl_free_xml_encoded_strs(uint32_t num_strs, char** dsts, bool* encoded_flags){
	uint32_t i;
        for (i = 0; i < num_strs; i++){
		if (!encoded_flags[i]){
			continue;
		}
		CMAPL_FREE(dsts[i]);
	}
}

cmapl_error_t
cmapl_get_period_index(uint64_t period, uint32_t* period_index)
{
	uint32_t index;
	for (index = 0; index < CMAPL_MAX_NUM_PERIODS; index++){
		if (_cmapl_period_map[index] == period){
			*period_index = index;
			return CMAPL_SUCCESS;
		}
	}
	return CMAPL_PERIOD_NOT_FOUND;
}

cmapl_error_t
cmapl_get_object_period_offset(cmapl_header_handle_t* header, uint32_t obj_count, uint64_t period, cmapl_offset_t* offset)
{
	if (header == NULL){
		return CMAPL_NULL_POINTER;
	}
	if (obj_count >= header->i6_0.num_objects_sampled){
		return CMAPL_OBJECT_NOT_FOUND;
	}
	uint32_t period_index = 0;
	cmapl_error_t err = CMAPL_SUCCESS;
	err = cmapl_get_period_index(period, &period_index);
	if (err != CMAPL_SUCCESS){
		return err;
	}

	uint32_t index = (obj_count * CMAPL_MAX_NUM_PERIODS) + period_index;
	*offset = header->i6_0.object_index[index];

	return err;
}

void
cmapl_get_version(void* data, uint32_t * major_v, uint32_t * minor_v)
{
	cmapl_version_t* ver = (cmapl_version_t*)data;

	*major_v = ver->major_v;
	*minor_v = ver->minor_v;
}

bool
cmapl_is_period_valid(uint64_t period)
{
	uint32_t index;
        for (index = 0; index < CMAPL_MAX_NUM_PERIODS; index++){
                if (_cmapl_period_map[index] == period){
                        return true;
                }
        }
        return false;
}

bool
cmapl_is_version(void* data, uint32_t major_v, uint32_t minor_v)
{
	cmapl_version_t* ver = (cmapl_version_t*)data;

	if ((ver->major_v == major_v) && (ver->minor_v == minor_v)){
		return true;
	} else {
		return false;
	}
}

cmapl_error_t
cmapl_parse_filename(char * filename, uint32_t * file_index, uint64_t * period, uint64_t * start_time, uint64_t * end_time)
{
	*file_index = 0;
	*start_time = 0;
        *end_time = 0;

	// parse the index, start time, and duration from the filename
	uint32_t index = 0;
	uint64_t max_period = 0;
	uint64_t start = 0;
	uint64_t duration = 0;
	int num = CMAPL_SSCANF(filename, "%"PRIu32"_%"PRIu64"_%"PRIu64"_%"PRIu64".", &index, &max_period, &start, &duration);
	if (num != 4){
		return CMAPL_FILENAME_FORMAT_ERROR;
	}

	// assign the index, period, and start and end time of the file
	*file_index = index;
	*period = max_period * 1000;
	*start_time = start;
	*end_time = start + duration;

	// validate values
        if (index < 1 ||
                !cmapl_is_period_valid(*period) ||
		*start_time >= *end_time
        ){
                return CMAPL_FILENAME_FORMAT_ERROR;
        }

        return CMAPL_SUCCESS;
}

void
cmapl_set_latest_version(void* data)
{
	cmapl_version_t * ver = (cmapl_version_t*)data;

	ver->major_v = CMAPL_CURRENT_MAJOR_VERSION;
	ver->minor_v = CMAPL_CURRENT_MINOR_VERSION;
}

cmapl_error_t
cmapl_set_object_period_offset(cmapl_header_handle_t* header, uint32_t obj_count, uint64_t period, cmapl_offset_t offset)
{
        if (header == NULL){
                return CMAPL_NULL_POINTER;
        }
        if (obj_count >= header->i6_0.num_objects_sampled){
                return CMAPL_OBJECT_NOT_FOUND;
        }
        uint32_t period_index = 0;
        cmapl_error_t err = CMAPL_SUCCESS;
        err = cmapl_get_period_index(period, &period_index);
        if (err != CMAPL_SUCCESS){
                return err;
        }

        uint32_t index = (obj_count * CMAPL_MAX_NUM_PERIODS) + period_index;
        header->i6_0.object_index[index] = offset;

        return err;
}

cmapl_error_t
cmapl_validate_attr_def_v6_0(cmapl_attribute_def_t_v6_0 * attr_def)
{
	// check if pointer fields are NULL
	if (attr_def == NULL ||
		attr_def->name == NULL ||
		attr_def->desc == NULL)
	{
		return CMAPL_NULL_POINTER;
	}

	return CMAPL_SUCCESS;
}

cmapl_error_t
cmapl_validate_counter_def_v6_0(cmapl_counter_def_t_v6_0 * counter_def)
{
	// check if pointer fields are NULL
	if (counter_def == NULL ||
                counter_def->name == NULL ||
                counter_def->desc == NULL ||
		counter_def->priv == NULL)
        {
                return CMAPL_NULL_POINTER;
        }

	// check if any x_name label is NULL
	if (counter_def->num_x_items > 0){
		if (counter_def->x_names == NULL){
			return CMAPL_NULL_POINTER;
		}

		uint32_t i;
		for (i = 0; i < counter_def->num_x_items; i++){
			if (counter_def->x_names[i] == NULL){
				return CMAPL_NULL_POINTER;
			}
		}
	}

	// check if any y_name label is NULL
	if (counter_def->num_y_items > 0){
                if (counter_def->y_names == NULL){
                        return CMAPL_NULL_POINTER;
                }

                uint32_t i;
                for (i = 0; i < counter_def->num_y_items; i++){
                        if (counter_def->y_names[i] == NULL){
                                return CMAPL_NULL_POINTER;
                        }
                }
        }

        return CMAPL_SUCCESS;
}

cmapl_error_t
cmapl_validate_directory_def_v6_0(cmapl_directory_def_t_v6_0 * dir_def)
{
	// check if pointer fields are NULL
	if (dir_def == NULL ||
                dir_def->directory_path == NULL)
        {
                return CMAPL_NULL_POINTER;
        }

	return CMAPL_SUCCESS;
}

cmapl_error_t
cmapl_validate_header_def_v6_0(cmapl_header_def_t_v6_0 * header_def)
{
	// check if pointer fields are NULL
        if (header_def == NULL ||
                header_def->preset_version == NULL)
        {
                return CMAPL_NULL_POINTER;
        }

	return CMAPL_SUCCESS;
}

cmapl_error_t
cmapl_validate_object_def_v6_0(cmapl_object_def_t_v6_0 * object_def)
{
	// check if pointer fields are NULL
        if (object_def == NULL ||
                object_def->name == NULL)
        {
                return CMAPL_NULL_POINTER;
        }

	return CMAPL_SUCCESS;
}

cmapl_error_t
cmapl_validate_query_def_v6_0(cmapl_query_def_t_v6_0 * query_def)
{
	// check if pointer fields are NULL
        if (query_def == NULL ||
                query_def->object_name == NULL)
        {
                return CMAPL_NULL_POINTER;
        }

	// check if key counter names are NULL
	if (query_def->num_key_cnts > 0){
		if (query_def->key_cnt_names == NULL){
			return CMAPL_NULL_POINTER;
		}

		uint32_t i;
		for (i = 0; i < query_def->num_key_cnts; i++){
			if (query_def->key_cnt_names[i] == NULL){
				return CMAPL_NULL_POINTER;
			}
		}
	}

	// check if counter names are NULL
        if (query_def->num_cnts > 0){
                if (query_def->cnt_names == NULL){
                        return CMAPL_NULL_POINTER;
                }

                uint32_t i;
                for (i = 0; i < query_def->num_cnts; i++){
                        if (query_def->cnt_names[i] == NULL){
                                return CMAPL_NULL_POINTER;
                        }
                }
        }

	// check if query period is valid
        if (!cmapl_is_period_valid(query_def->period)){
                return CMAPL_PERIOD_NOT_FOUND;
        }

	// check if start and end times are valid
	if (query_def->start_time > query_def->end_time){
		return CMAPL_INVALID_TIMESTAMP;
	}

	return CMAPL_SUCCESS;
}

cmapl_error_t
cmapl_validate_system_def_v6_0(cmapl_system_def_t_v6_0 * system_def)
{
	// check if pointer fields are NULL
        if (system_def == NULL ||
                system_def->ontap_version == NULL ||
		system_def->node_uuid == NULL ||
		system_def->hostname == NULL ||
		system_def->model == NULL ||
		system_def->serial_number == NULL ||
		system_def->system_id == NULL)
        {
                return CMAPL_NULL_POINTER;
        }

	return CMAPL_SUCCESS;
}

void
cmapl_free_counter_data(cmapl_counter_data_handle_t* counter_data)
{
	if (counter_data == NULL){
                return;
        }

	if (cmapl_is_version(counter_data, 5, 0)){
		// free external struct
		if (counter_data->e5_0.value_type == CMAPL_CNT_STRING){
			uint32_t i;
			for (i = 0; i < counter_data->e5_0.num_values; i++){
				CMAPL_FREE(counter_data->e5_0.values[i].val_str);
			}
		}
		CMAPL_FREE(counter_data->e5_0.values);

		// the following pointers don't need to be freed
		counter_data->i5_0.counter = NULL;
		counter_data->i5_0.instance = NULL;
	} else if (cmapl_is_version(counter_data, 6, 0)){
		// free external struct
		if (counter_data->e6_0.value_type == CMAPL_CNT_STRING && counter_data->e6_0.values){
			uint32_t i;
                        for (i = 0; i < counter_data->e6_0.num_values; i++){
				CMAPL_FREE(counter_data->e6_0.values[i].val_str);
			}
                }
		CMAPL_FREE(counter_data->e6_0.values);

		// the following pointers don't need to be freed
		counter_data->i6_0.counter = NULL;
                counter_data->i6_0.instance = NULL;
	} else{
		CMAPL_ASSERT(0);
	}

	// free handle
	CMAPL_FREE(counter_data);
}

void
cmapl_free_instance(cmapl_instance_handle_t* instance)
{
	if (instance == NULL){
                return;
        }

	if (cmapl_is_version(instance, 5, 0)){
		// free external struct
		CMAPL_FREE(instance->e5_0.name);

                // free internal struct
                cmapl_free_counter_data(instance->i5_0.current_counter_data);

                // the following pointers don't need to be freed
                instance->i5_0.timestamp = NULL;
		instance->i5_0.instance_buffer = NULL;
        } else if (cmapl_is_version(instance, 6, 0)){
		// free external struct
		uint32_t i;
		for (i = 0; i < instance->e6_0.num_key_cnts; i++){
                        if (instance->e6_0.key_cnt_names) {
                                CMAPL_FREE(instance->e6_0.key_cnt_names[i]);
                        }
			if (instance->e6_0.key_cnt_values &&
			    instance->e6_0.key_cnt_values[i].value_type == CMAPL_CNT_STRING) {
				uint32_t j;
				for (j = 0; j < instance->e6_0.key_cnt_values[i].num_values; j++){
					CMAPL_FREE(instance->e6_0.key_cnt_values[i].values[j].val_str);
				}
				CMAPL_FREE(instance->e6_0.key_cnt_values[i].values);
			}
		}
		CMAPL_FREE(instance->e6_0.key_cnt_names);
		CMAPL_FREE(instance->e6_0.key_cnt_values);

                // free internal struct
		cmapl_free_counter_data(instance->i6_0.current_counter_data);

                // the following pointers don't need to be freed
                instance->i6_0.timestamp = NULL;
		instance->i6_0.instance_buffer = NULL;
        } else{
                CMAPL_ASSERT(0);
        }

        // free handle
        CMAPL_FREE(instance);
}

void
cmapl_free_timestamp(cmapl_timestamp_handle_t* timestamp)
{
	if (timestamp == NULL){
		return;
	}
	if (cmapl_is_version(timestamp, 5, 0)){
		// free internal struct
		CMAPL_FREE(timestamp->i5_0.instances_buffer);
		cmapl_free_instance(timestamp->i5_0.current_instance);

		// the following pointers don't need to be freed
		timestamp->i5_0.query = NULL;
	} else if (cmapl_is_version(timestamp, 6, 0)){
		// free internal struct
		CMAPL_FREE(timestamp->i6_0.instances_buffer);
		cmapl_free_instance(timestamp->i6_0.current_instance);

		// the following pointers don't need to be freed
		timestamp->i6_0.query = NULL;
	} else{
		CMAPL_ASSERT(0);
	}

	// free handle
	CMAPL_FREE(timestamp);
}

void
cmapl_free_query(cmapl_query_handle_t* query)
{
	if (query == NULL){
                return;
        }
	if (cmapl_is_version(query, 5, 0)){
		// free external struct
		CMAPL_FREE(query->e5_0.object_name);
		CMAPL_FREE(query->e5_0.instance_name);

                // free internal struct
                CMAPL_FREE(query->i5_0.timestamps);
                CMAPL_FREE(query->i5_0.counters);
		cmapl_free_timestamp(query->i5_0.current_timestamp);

                // the following pointers don't need to be freed
                query->i5_0.data_file = NULL;
	} else if (cmapl_is_version(query, 6, 0)){
		// free external struct
		CMAPL_FREE(query->e6_0.object_name);
		uint32_t i;
		for(i = 0; i < query->e6_0.num_key_cnts; i++){
                        if (query->e6_0.key_cnt_names) {
                                CMAPL_FREE(query->e6_0.key_cnt_names[i]);
                        }
			if (query->e6_0.key_cnt_values &&
			    query->e6_0.key_cnt_values[i].value_type == CMAPL_CNT_STRING) {
				uint32_t j;
                                for (j = 0; j < query->e6_0.key_cnt_values[i].num_values; j++){
					CMAPL_FREE(query->e6_0.key_cnt_values[i].values[j].val_str);
				}
				CMAPL_FREE(query->e6_0.key_cnt_values[i].values);
			}
		}
		CMAPL_FREE(query->e6_0.key_cnt_names);
		CMAPL_FREE(query->e6_0.key_cnt_values);

                if (query->e6_0.cnt_names) {
                        for(i = 0; i < query->e6_0.num_cnts; i++){
                                CMAPL_FREE(query->e6_0.cnt_names[i]);
                        }
                }
		CMAPL_FREE(query->e6_0.cnt_names);

		// free internal struct
		CMAPL_FREE(query->i6_0.timestamps);
		CMAPL_FREE(query->i6_0.counters);
		CMAPL_FREE(query->i6_0.key_counters);
		CMAPL_FREE(query->i6_0.periods);
		cmapl_free_timestamp(query->i6_0.current_timestamp);

		// the following pointers don't need to be freed
		query->i6_0.data_file = NULL;
	} else{
		CMAPL_ASSERT(0);
	}

	// free handle
	CMAPL_FREE(query);
}

void
cmapl_free_record(cmapl_record_handle_t* record)
{
	if (record == NULL){
		return;
	}
	if (cmapl_is_version(record, 5, 0)){
		// free internal struct
		CMAPL_FREE(record->i5_0.data_buffer);
	} else if (cmapl_is_version(record, 6, 0)){
		// free internal struct
		CMAPL_FREE(record->i6_0.data_buffer);
                CMAPL_FREE(record->i6_0.counters);
                cmapl_free_instance(record->i6_0.current_instance);

                // the following pointers don't need to be freed
                record->i6_0.object = NULL;
                record->i6_0.period = NULL;
	} else{
		CMAPL_ASSERT(0);
	}

	// free handle
	CMAPL_FREE(record);
}

void
cmapl_free_counter(cmapl_counter_handle_t* counter)
{
	if(counter == NULL){
		return;
	}
	if (cmapl_is_version(counter, 5, 0)){
		// free external struct
                CMAPL_FREE(counter->e5_0.name);
                CMAPL_FREE(counter->e5_0.desc);

                uint32_t i;
                for(i = 0; i < counter->e5_0.num_x_items; i++){
                        CMAPL_FREE(counter->e5_0.x_names[i]);
                }
                CMAPL_FREE(counter->e5_0.x_names);

                for(i = 0; i < counter->e5_0.num_y_items; i++){
                        CMAPL_FREE(counter->e5_0.y_names[i]);
                }
                CMAPL_FREE(counter->e5_0.y_names);

                // free internal struct
                // the following pointers don't need to be freed
                counter->i5_0.next_cnt_by_id = NULL;
		counter->i5_0.object = NULL;
	}else if (cmapl_is_version(counter, 6, 0)){
		// free external struct
		CMAPL_FREE(counter->e6_0.name);
		CMAPL_FREE(counter->e6_0.priv);
		CMAPL_FREE(counter->e6_0.desc);

		uint32_t i;
                if (counter->e6_0.x_names) {
                        for(i = 0; i < counter->e6_0.num_x_items; i++){
                                CMAPL_FREE(counter->e6_0.x_names[i]);
                        }
                        CMAPL_FREE(counter->e6_0.x_names);
                }

                if (counter->e6_0.y_names) {
                        for(i = 0; i < counter->e6_0.num_y_items; i++){
                                CMAPL_FREE(counter->e6_0.y_names[i]);
                        }
                        CMAPL_FREE(counter->e6_0.y_names);
                }

		// free internal struct
		// the following pointers don't need to be freed
		counter->i6_0.next_cnt_by_id = NULL;
		counter->i6_0.next_cnt_by_key = NULL;
		counter->i6_0.object = NULL;
		for(i = 0; i < CMAPL_MAX_NUM_PERIODS; i++){
			counter->i6_0.periods[i] = NULL;
			counter->i6_0.next_cnt_by_period[i] = NULL;
		}
	}else{
                CMAPL_ASSERT(0);
        }

	// free handle
	CMAPL_FREE(counter);
}

void
cmapl_free_period(cmapl_period_t* period_list)
{
	if(period_list == NULL){
                return;
        }

	cmapl_period_t* current_period;

	while(period_list != NULL){
		current_period = period_list;
		period_list = period_list->next_period;

		CMAPL_FREE(current_period->record_offsets);
                if (current_period->iterative_record_offsets) {
                        uint32_t i;
                        for(i = 0; i < current_period->num_offsets; i++){
                                // NOTE: defensive code added to prevent panic - burt 688768
                                if (current_period->iterative_record_offsets[i].num_offsets == 0) {
                                        continue;
                                }
                                CMAPL_FREE(current_period->iterative_record_offsets[i].offsets);
                        }
                }
		CMAPL_FREE(current_period->iterative_record_offsets);

		// the following pointers don't need to be freed
		current_period->cnt_list = NULL;

		CMAPL_FREE(current_period);
	}
}

void
cmapl_free_object(cmapl_object_handle_t* object)
{
	if(object == NULL){
                return;
        }

	if (cmapl_is_version(object, 5, 0)){
		// free external struct
                CMAPL_FREE(object->e5_0.name);

                // free internal struct
                cmapl_counter_handle_t* counter_list = object->i5_0.cnt_list;
                cmapl_counter_handle_t* current_counter;

                while(counter_list != NULL){
                        current_counter = counter_list;
                        counter_list = counter_list->i5_0.next_cnt_by_id;
                        cmapl_free_counter(current_counter);
                }

		// the following pointers don't need to be freed
                object->i5_0.header = NULL;
	}else if (cmapl_is_version(object, 6, 0)){
		// free external struct
		CMAPL_FREE(object->e6_0.name);

		// free internal struct
		cmapl_counter_handle_t* counter_list = object->i6_0.cnt_list;
		cmapl_counter_handle_t* current_counter;

		while(counter_list != NULL){
			current_counter = counter_list;
			counter_list = counter_list->i6_0.next_cnt_by_id;
			cmapl_free_counter(current_counter);
		}

		cmapl_free_period(object->i6_0.period_list);

		// the following pointers don't need to be freed
		object->i6_0.key_cnt_list = NULL;
		object->i6_0.next_object = NULL;
		object->i6_0.header = NULL;
	}else{
		CMAPL_ASSERT(0);
	}

	// free handle
	CMAPL_FREE(object);
}

void
cmapl_free_system(cmapl_header_system_t syst)
{
	// free external struct
	if (cmapl_is_version(&syst, 5, 0)){
                CMAPL_FREE(syst.e5_0.ontap_version);
                CMAPL_FREE(syst.e5_0.local_time);
		CMAPL_FREE(syst.e5_0.gm_time);
                CMAPL_FREE(syst.e5_0.hostname);
                CMAPL_FREE(syst.e5_0.model);
                CMAPL_FREE(syst.e5_0.serial_number);
                CMAPL_FREE(syst.e5_0.system_id);
	}else if(cmapl_is_version(&syst, 6, 0)){
                CMAPL_FREE(syst.e6_0.ontap_version);
                CMAPL_FREE(syst.e6_0.node_uuid);
                CMAPL_FREE(syst.e6_0.hostname);
                CMAPL_FREE(syst.e6_0.model);
                CMAPL_FREE(syst.e6_0.serial_number);
                CMAPL_FREE(syst.e6_0.system_id);
	}else{
		CMAPL_ASSERT(0);
	}
}

void
cmapl_free_attr_def(void* attr_def)
{
	if(attr_def == NULL){
                return;
        }

	if (cmapl_is_version(attr_def, 5, 0)){
		cmapl_attribute_def_t_v5_0* attr = (cmapl_attribute_def_t_v5_0*)attr_def;
                CMAPL_FREE(attr->name);
                CMAPL_FREE(attr->desc);
		CMAPL_FREE(attr->frequency);
	}else if (cmapl_is_version(attr_def, 6, 0)){
		cmapl_attribute_def_t_v6_0* attr = (cmapl_attribute_def_t_v6_0*)attr_def;
		CMAPL_FREE(attr->name);
		CMAPL_FREE(attr->desc);
	}else{
		CMAPL_ASSERT(0);
	}

	CMAPL_FREE(attr_def);
}

void
cmapl_free_header(cmapl_header_handle_t* header)
{
	if(header == NULL){
                return;
        }

	if (cmapl_is_version(header, 5, 0)){
                // free internal struct
                cmapl_free_system(header->i5_0.system);

                uint32_t i;
                for(i = 0; i< header->i5_0.attr_array_size; i++){
                        cmapl_free_attr_def(header->i5_0.attr_defs[i]);
                }
                CMAPL_FREE(header->i5_0.attr_defs);

                for(i = 0; i< header->i5_0.objects_array_size; i++){
                        cmapl_object_handle_t* object = header->i5_0.objects[i];
                        if(object != NULL){
                                cmapl_free_object(object);
                        }
                }
                CMAPL_FREE(header->i5_0.objects);

		// the following pointers don't need to be freed
                header->i5_0.data_file = NULL;
	}else if (cmapl_is_version(header, 6, 0)){
		// free external struct
		CMAPL_FREE(header->e6_0.preset_version);

		// free internal struct
		cmapl_free_system(header->i6_0.system);

		uint32_t i;
                if (header->i6_0.attr_defs) {
                        for(i = 0; i< header->i6_0.num_attr_defs; i++){
                                cmapl_free_attr_def(header->i6_0.attr_defs[i]);
                        }
                }
		CMAPL_FREE(header->i6_0.attr_defs);
		CMAPL_FREE(header->i6_0.section_defs);

                if (header->i6_0.objects) {
                        for(i = 0; i< header->i6_0.objects_array_size; i++){
                                cmapl_object_handle_t* object = header->i6_0.objects[i];
                                if(object != NULL){
                                        cmapl_free_object(object);
                                }
                        }
                }
		CMAPL_FREE(header->i6_0.objects);
		CMAPL_FREE(header->i6_0.object_index);
#ifdef _KERNEL
		RETURN_ADDRESS(header->i6_0.pc);
		header->i6_0.pid = sk_my_pid();
#endif /* _KERNEL */

		// the following pointers don't need to be freed
		header->i6_0.object_list = NULL;
		header->i6_0.data_file = NULL;
	}else{
                CMAPL_ASSERT(0);
        }

	// free handle
	CMAPL_FREE(header);
}

cmapl_error_t
cmapl_free_file(cmapl_file_handle_t* data_file)
{
	if(data_file == NULL){
                return CMAPL_SUCCESS;
        }

	cmapl_error_t err = CMAPL_SUCCESS;
	if (cmapl_is_version(data_file, 5, 0)){
		// free external struct
		CMAPL_FREE(data_file->e5_0.file_path);

		// file handles must be closed first
		if(data_file->i5_0.data_file != NULL){
                        CMAPL_FCLOSE(data_file->i5_0.data_file);
			data_file->i5_0.data_file = NULL;
                }

		// free internal struct
		CMAPL_FREE(data_file->i5_0.file_path);
                cmapl_free_header(data_file->i5_0.header);
		cmapl_free_query(data_file->i5_0.query);
	}else if (cmapl_is_version(data_file, 6, 0)){
		// free external struct
                CMAPL_FREE(data_file->e6_0.file_path);
		CMAPL_FREE(data_file->e6_0.meta_path);
		CMAPL_FREE(data_file->e6_0.data_path);

		// file handle must be closed first
		if(data_file->i6_0.metadata_file != NULL){
			CMAPL_FCLOSE(data_file->i6_0.metadata_file);
			data_file->i6_0.metadata_file = NULL;
		}
		if(data_file->i6_0.data_file != NULL){
			int num = CMAPL_GZCLOSE(data_file->i6_0.data_file);
			if (num != 0){
				err = CMAPL_CLOSE_FILE_FAILED;
			}
			data_file->i6_0.data_file = NULL;
		}

		// free internal struct
		CMAPL_FREE(data_file->i6_0.metadata_file_path);
		CMAPL_FREE(data_file->i6_0.data_file_path);
		cmapl_free_header(data_file->i6_0.header);
		cmapl_free_query(data_file->i6_0.query);
	}else{
		CMAPL_ASSERT(0);
	}

	// free handle
	CMAPL_FREE(data_file);
	return err;
}

void
cmapl_free_directory(cmapl_directory_handle_t* dir)
{
	if(dir == NULL){
                return;
        }

	if (cmapl_is_version(dir, 5, 0)){
		// free external struct
                CMAPL_FREE(dir->e5_0.directory_path);

                // free internal struct
                uint32_t i;
                uint32_t num_data_files = dir->i5_0.num_data_files;
                for (i = 0; i < num_data_files; i++){
			CMAPL_FREE(dir->i5_0.file_infos[i].file_path);
                }
                CMAPL_FREE(dir->i5_0.file_infos);
        }else if (cmapl_is_version(dir, 6, 0)){
		// free external struct
		CMAPL_FREE(dir->e6_0.directory_path);

                // free internal struct
		uint32_t i;
		uint32_t num_files = dir->i6_0.num_files;
                if (dir->i6_0.file_infos) {
                        for (i = 0; i < num_files; i++){
                                CMAPL_FREE(dir->i6_0.file_infos[i].file_path);
                                CMAPL_FREE(dir->i6_0.file_infos[i].meta_path);
                                CMAPL_FREE(dir->i6_0.file_infos[i].data_path);
                        }
                }
		CMAPL_FREE(dir->i6_0.file_infos);
        }else{
                CMAPL_ASSERT(0);
        }

        // free handle
        CMAPL_FREE(dir);
}

#ifdef _CMAPL_USER_SPACE_
void
cmapl_free_corefile(cmapl_corefile_handle_t* core)
{
        if (core == NULL) {
                return;
        }

        if (cmapl_is_version(core, 6, 0)) {
                // free external struct
                CMAPL_FREE(core->e6_0.file_path);

                // free internal struct
                // free and remove lzma data
#ifdef _WIN32
                if (core->i6_0.archiveStream.file.handle != NULL) {
#else
                if (core->i6_0.archiveStream.file.file != NULL) {
#endif
                        SzArEx_Free(&(core->i6_0.db), &(core->i6_0.allocImp));
                        (void)File_Close(&(core->i6_0.archiveStream.file));
                }

                uint32_t i;
                uint32_t num_files = core->i6_0.num_files;
                if (core->i6_0.file_names) {
                        for (i = 0; i < num_files; i++) {
                                CMAPL_FREE(core->i6_0.file_names[i]);
                        }
                }
                CMAPL_FREE(core->i6_0.file_names);

                uint32_t num_dirs = core->i6_0.num_dirs;
                if (core->i6_0.dir_names) {
                        for (i = 0; i < num_dirs; i++) {
                                CMAPL_FREE(core->i6_0.dir_names[i]);
                        }
                }
                CMAPL_FREE(core->i6_0.dir_names);

                CMAPL_FREE(core->i6_0.temp_dir);
        } else {
                CMAPL_ASSERT(0);
        }

        // free handle
        CMAPL_FREE(core);
}
#endif /* _CMAPL_USER_SPACE_ */
