﻿using NUnit.Framework;
using odct;
using odct.Clusters;

namespace odct.tests {

    [TestFixture]
    [Category("Integration")]
    class ClusterTests {

        public Credentials credentials;
        public Cluster cluster;

        [TestFixtureSetUp]
        public void init() {
            credentials = new Credentials("10.26.9.240", "admin", "netapp11");
            credentials.Verify(new MessengerStub());
            cluster = new Cluster(credentials, new MessengerStub(), "");
        }

        [Test]
        [Category("Integration")]
        public void getClusterIdentityInfoIntegrationTests() {

            Assert.True("".Equals(cluster.clusterContact));
            Assert.True("Dallas CSE Lab".Equals(cluster.clusterLocation));
            Assert.True("atlas".Equals(cluster.clusterName));
            Assert.True("1-80-000011".Equals(cluster.clusterSerialNumber));
            Assert.True("b2f58e7c-0109-11e3-9509-123478563412".Equals(cluster.clusterUuid));
            Assert.True("b2f58e7c-0109-11e3-9509-123478563412".Equals(cluster.clusterRdbUuid));
        }

        [Test]
        [Category("Integration")]
        public void getNodesIntegrationTests() {

            Credentials credentials = new Credentials("10.26.9.240", "admin", "netapp11");
            credentials.Verify(new MessengerStub());
            Cluster cluster = new Cluster(credentials, new MessengerStub(), "");

            Assert.AreEqual(4, cluster.nodeNamesToNodes.Count);
            Assert.True(cluster.nodeNamesToNodes.ContainsKey("atlas-01"));
            Assert.True(cluster.nodeNamesToNodes.ContainsKey("atlas-02"));
            Assert.True(cluster.nodeNamesToNodes.ContainsKey("atlas-03"));
            Assert.True(cluster.nodeNamesToNodes.ContainsKey("atlas-04"));

            Assert.AreEqual("10.26.9.240", cluster.clusterMgmtIp);
            Assert.AreEqual("atlas-01", cluster.clusterHomeNode);

            ClusterNode atlas1 = cluster.nodeNamesToNodes["atlas-01"];
            Assert.AreEqual("10.26.9.241", atlas1.nodeMgmtIp);
            Assert.AreEqual("atlas-01", atlas1.nodeName);

        }
    
    }
}
