﻿using System;
using System.Collections.Generic;
using System.IO;
using NUnit.Framework;
using odct;
using odct.Clusters;

namespace odct.tests {
    [TestFixture]
    [Category("BrokenTests")]
    // These tests work from nunit-x86.exe
    public class CmaplCsvTests {

        private string getTestFilePath(string filename, string subdirectory = null) {
            if (!string.IsNullOrEmpty(subdirectory)) {
                return Path.Combine(
                    OdctSettingsTest.GetRelativeUnitTestDataDirectory(subdirectory),
                    filename);
            } else {
                return Path.Combine(
                    OdctSettingsTest.GetRelativeUnitTestDataDirectory(),
                    filename);
            }
        }

        [TestFixtureSetUp]
        public void init() {
            CmaplWrapper.LoadDllsIfNecessary();   // It seems like this should be necessary, but runs fine without it!
        }

        [Test]
        public void canGetCsv() {

            var infilePath = getTestFilePath("043311_003600_1462991400454_0899000.ccma", "cmode84");
            var outfilePath = Path.GetTempFileName();
            string objectName = "system";
            Int32 numObjects = 1;
            string countersString = "system_ops";
            string numCounters = "1";
            Int32 frequency = 60000;

            CmaplWrapper.GetCsv(
				infilePath,
                outfilePath,
				objectName,
				numObjects,
				countersString,
				numCounters,
				frequency,
                new MessengerStub(),
                "");

            var result = File.ReadAllText(outfilePath);
            Console.WriteLine(result);

            var expected = "TIMESTAMP,instance_name,system:system_ops\n"
                                + "1462991448454,gorgon-03,504075684\n"
                                + "1462991508454,gorgon-03,504080962\n"
                                + "1462991568454,gorgon-03,504085189\n"
                                + "1462991628454,gorgon-03,504090393\n"
                                + "1462991688454,gorgon-03,504094565\n"
                                + "1462991748454,gorgon-03,504098802\n"
                                + "1462991808454,gorgon-03,504103661\n"
                                + "1462991868454,gorgon-03,504108369\n"
                                + "1462991928454,gorgon-03,504112676\n"
                                + "1462991988454,gorgon-03,504123966\n"
                                + "1462992048454,gorgon-03,504128125\n"
                                + "1462992108454,gorgon-03,504133665\n"
                                + "1462992168454,gorgon-03,504137807\n"
                                + "1462992228454,gorgon-03,504142059\n"
                                + "1462992288454,gorgon-03,504146269\n";

            Assert.AreEqual(expected, result);
        }

        [Test]
        public void canGetMultipleCountersCsv() {

            var infilePath = getTestFilePath("043311_003600_1462991400454_0899000.ccma", "cmode84");
            var outfilePath = Path.GetTempFileName();
            string objectName = "system";
            Int32 numObjects = 1;
            string countersString = "system_ops,cifs_ops";
            string numCounters = "2";
            Int32 frequency = 60000;

            CmaplWrapper.GetCsv(
                infilePath,
                outfilePath,
                objectName,
                numObjects,
                countersString,
                numCounters,
                frequency,
                new MessengerStub(),
                "");

            var result = File.ReadAllText(outfilePath);
            Console.WriteLine(result);

            var expected = "TIMESTAMP,instance_name,system:system_ops,system:cifs_ops\n"
                + "1462991448454,gorgon-03,504075684,0\n"
                + "1462991508454,gorgon-03,504080962,0\n"
                + "1462991568454,gorgon-03,504085189,0\n"
                + "1462991628454,gorgon-03,504090393,0\n"
                + "1462991688454,gorgon-03,504094565,0\n"
                + "1462991748454,gorgon-03,504098802,0\n"
                + "1462991808454,gorgon-03,504103661,0\n"
                + "1462991868454,gorgon-03,504108369,0\n"
                + "1462991928454,gorgon-03,504112676,0\n"
                + "1462991988454,gorgon-03,504123966,0\n"
                + "1462992048454,gorgon-03,504128125,0\n"
                + "1462992108454,gorgon-03,504133665,0\n"
                + "1462992168454,gorgon-03,504137807,0\n"
                + "1462992228454,gorgon-03,504142059,0\n"
                + "1462992288454,gorgon-03,504146269,0\n";

            Assert.AreEqual(expected, result);
        }

        [Test]
        public void canGetHistogramCsv() {

            var infilePath = getTestFilePath("043311_003600_1462991400454_0899000.ccma", "cmode84");
            var outfilePath = Path.GetTempFileName();
            string objectName = "system";
            Int32 numObjects = 1;
            string countersString = "domain_busy";
            string numCounters = "1";
            Int32 frequency = 300000;

            CmaplWrapper.GetCsv(
                infilePath,
                outfilePath,
                objectName,
                numObjects,
                countersString,
                numCounters,
                frequency,
                new MessengerStub(),
                "");

            var result = File.ReadAllText(outfilePath);
            Console.WriteLine(result);

            var expected = "TIMESTAMP,instance_name,"
                + "system:domain_busy:idle,"
                + "system:domain_busy:kahuna,"
                + "system:domain_busy:storage,"
                + "system:domain_busy:exempt,"
                + "system:domain_busy:raid,"
                + "system:domain_busy:raid_exempt,"
                + "system:domain_busy:xor_exempt,"
                + "system:domain_busy:target,"
                + "system:domain_busy:wafl_exempt,"
                + "system:domain_busy:wafl_mpcleaner,"
                + "system:domain_busy:sm_exempt,"
                + "system:domain_busy:protocol,"
                + "system:domain_busy:nwk_exclusive,"
                + "system:domain_busy:nwk_exempt,"
                + "system:domain_busy:nwk_legacy,"
                + "system:domain_busy:hostOS,"
                + "system:domain_busy:ssan_exempt,"
                + "system:domain_busy:unclassified,"
                + "system:domain_busy:kahuna_legacy,"
                + "system:domain_busy:ha\n"
                + "1462991448454,gorgon-03,2004425879599,3753195086,738992179,11819200793,182724601,279878915,167413333,7098008,22011759840,1638177423,14662194,229113946,949017385,5159312170,1254215591,28990544404,14918913828,5570018,7773842,124603916\n"
                + "1462991748454,gorgon-03,2006764488743,3757373883,739630555,11830415927,182907974,280060736,167503888,7105948,22013482495,1638345504,14676465,229360745,949949006,5164541390,1255432471,29021449499,14919361507,5574135,7777043,124741218\n"
                + "1462992048454,gorgon-03,2009110985831,3761643360,740330346,11841968486,183126439,280261362,167604962,7115109,22015397835,1638519982,14691697,229621959,951038820,5169871362,1256923027,29043087796,14919812891,5582766,7780324,124887081\n";

            Assert.AreEqual(expected, result);
        }

        [Test]
        [Category("BrokenTests")]
        [Ignore("Should be re-written.")]
        public void canGetCluster_8_1Csv() {
            // TODO: this needs rework; it is written to run like a c-mode collection
            //    but 8.1 c-mode archives are more like 7-mode.
            var infilePath = getTestFilePath("stats_archive_data_1468925789_20160719_105629.gz", "cmode81");
            var outfilePath = Path.GetTempFileName();
            string objectName = "system";
            Int32 numObjects = 1;
            string countersString = "cifs_ops";
            string numCounters = "1";
            Int32 frequency = 1000;

            CmaplWrapper.GetCsv(
                infilePath,
                outfilePath,
                objectName,
                numObjects,
                countersString,
                numCounters,
                frequency,
                new MessengerStub(),
                "");

            var result = File.ReadAllText(outfilePath);
            Console.WriteLine(result);

            var expected = "TIMESTAMP,instance_name,system:system_ops,system:cifs_ops\n"
                + "1462991448454,gorgon-03,504075684,0\n"
                + "1462991508454,gorgon-03,504080962,0\n"
                + "1462991568454,gorgon-03,504085189,0\n"
                + "1462991628454,gorgon-03,504090393,0\n"
                + "1462991688454,gorgon-03,504094565,0\n"
                + "1462991748454,gorgon-03,504098802,0\n"
                + "1462991808454,gorgon-03,504103661,0\n"
                + "1462991868454,gorgon-03,504108369,0\n"
                + "1462991928454,gorgon-03,504112676,0\n"
                + "1462991988454,gorgon-03,504123966,0\n"
                + "1462992048454,gorgon-03,504128125,0\n"
                + "1462992108454,gorgon-03,504133665,0\n"
                + "1462992168454,gorgon-03,504137807,0\n"
                + "1462992228454,gorgon-03,504142059,0\n"
                + "1462992288454,gorgon-03,504146269,0\n";

            Assert.AreEqual(expected, result);
        }


        [Test]
        public void doGetEmptyStringIfNoCountersFound() {

            var infilePath = getTestFilePath("043311_003600_1462991400454_0899000.ccma", "cmode84");
            var outfilePath = Path.GetTempFileName();
            string objectName = "system";
            Int32 numObjects = 1;
            string countersString = "bad_counter";
            string numCounters = "1";
            Int32 frequency = 300000;

            CmaplWrapper.GetCsv(
                infilePath,
                outfilePath,
                objectName,
                numObjects,
                countersString,
                numCounters,
                frequency,
                new MessengerStub(),
                "");

            var result = File.ReadAllText(outfilePath);
            Console.WriteLine(result);

            var expected = "";

            Assert.AreEqual(expected, result);
        }
    
    }
}
