﻿using System;
using System.Collections.Generic;
using System.IO;
using NUnit.Framework;
using odct;
using odct.Clusters;
using odct.Counters;
using odct.Settings;

namespace odct.tests.Counters {

    [TestFixture]
    [Category("Integration")]
    public class ArchiveParserTestsCluster_8_1 {

        string serialno;
        Messenger messenger;
        ArchiveFileSource source;
        Dictionary<string,int> requestedCounters;
        bool writexmltopayload;
        static string work = TestContext.CurrentContext.WorkDirectory;
        static string testDir = Path.Combine(work, @"\odcttest\parsertest");

        ArchiveParser_7_3 ap;

        [TestFixtureSetUp]
        public void init() {

            serialno = "";
            messenger = new MessengerStub();
            ZapiContextStub zc = new ZapiContextStub(null, null);
            source =  new ZapiArchiveFileSource(zc) {
                Filename = "stats_archive_data_1468925789_20160719_105629.gz",
                Path = OdctSettingsTest.GetRelativeUnitTestDataDirectory("cmode81")
            };
            requestedCounters = new Dictionary<string,int>();
            requestedCounters["system.cifs_ops"] = 1;
            writexmltopayload = false;

            ap = new ArchiveParser_7_3(serialno, messenger, source, "", requestedCounters, writexmltopayload);

        }

        [Test]
        public void canGet81ArchiveParser() {
            ap = new ArchiveParser_7_3(
                "",
                new MessengerStub(),
                source,
                testDir,
                requestedCounters,
                false);

            Assert.IsInstanceOf(typeof(ArchiveParser_7_3), ap);
        }

        [Test]
        public void canGetCluster81Counter() {
            requestedCounters["system.cifs_ops"] = 1;
            requestedCounters["system.nfs_ops"] = 1;
            requestedCounters["system.http_ops"] = 1;
            requestedCounters["system.iscsi_ops"] = 1;
            requestedCounters["system.fcp_ops"] = 1;

            requestedCounters["system.system_model"] = 0;
            requestedCounters["system.ontap_version"] = 0;
            requestedCounters["system.serial_no"] = 0;
            requestedCounters["system.hostname"] = 0;
            requestedCounters["system.system_id"] = 0;

            ap = new ArchiveParser_7_3(
                "",
                new MessengerStub(),
                source,
                testDir,
                requestedCounters,
                false);

            ap.processRecords();

//            Assert.AreEqual(1, ap.writeXMLAsFile());

        }

        [Test]
        [Ignore("Will not be used in production but was useful in development")]
        public void canCopyXmlToFile() {
            Assert.AreEqual(1, ap.writeXMLAsFile());
        }

        [Test]
        [Category("Unit")]
        public void canPassFormatToStringFormat() {
            string param1 = "ABC";
            string formatA = "{0}";
            string formatB = "xx{0}xx";

            Assert.AreEqual("ABC", string.Format(formatA, param1));
            Assert.AreEqual("xxABCxx", string.Format(formatB, param1));
        }

        [Test]
        [Category("BrokenTests")]
        [Ignore("Cannot be run without an 8.1 c-mode cluster")]
        public void canParse81ArchivesFromSourceGenerator() {
            Credentials credentials = new Credentials("Need creds", "from an", "8.1 cluster");
            credentials.Verify(new MessengerStub());
            ClusterNode cn = new ClusterNode(new MessengerStub(), new ClusterStub_8_3(), "nodename", credentials, "clusterpath");
            cn.nodeMgmtIp = "may need to set manually here...";
            Cluster81SourceGenerator clusterGen = new Cluster81SourceGenerator(cn);
            Assert.Less(100, clusterGen.Count());
        }

    }
}
