﻿using NUnit.Framework;
using System;
using System.IO;
using odct.Counters;
using odct.Settings;

namespace odct.tests.Counters {

    [TestFixture]
    class CounterDefinitionsTests {
        private string xmlContent;

        [TestFixtureSetUp]
        public void init() {
            var metaXmlPath = Path.Combine(
                OdctSettingsTest.GetRelativeUnitTestDataDirectory("ontap73"),
                "stats_archive_data_1447684431_20151116_093351.xml");

            this.xmlContent = File.ReadAllText(metaXmlPath);
        }

        private CounterDefinitions getTrtCounterDefinitions() {
            return new CounterDefinitions(
                            this.xmlContent,
                            OdctSettings.AppToPerfCounters("trt"));
        }

        [Test]
        [Category("Unit")]
        public void canGetCounterDetail() {
            var counterDefs = getTrtCounterDefinitions();
            var nfs_ops = counterDefs.GetCounterDefinitionByDottedName("system.nfs_ops");
            Assert.NotNull(nfs_ops);
        }

        [Test]
        [Category("Unit")]
        public void canGetObjectDetail() {
            var counterDefs = this.getTrtCounterDefinitions();
            var systemOid = counterDefs.GetOidByObjectName("system");
            Assert.AreEqual(0, systemOid);
        }

        [Test]
        [Category("Unit")]
        public void canGetAllCountersForObjects() {
            var counterDefs = new CounterDefinitions(
                this.xmlContent,
                new string[] { "system", "volume" } );

            var systemObj = counterDefs.GetCounterDefinitionLookup("system");
            var volObj = counterDefs.GetCounterDefinitionLookup("volume");
            //var raidObj = counterDefs.GetCounterDefinitionLookup("raid");

            Assert.NotNull(systemObj);
            Assert.NotNull(volObj);
            Assert.AreNotEqual(systemObj, volObj);
            Assert.AreNotEqual(systemObj.Oid, volObj.Oid);
        }
    }
}
