﻿using System;
using System.Collections.Generic;
using System.IO;
using NUnit.Framework;
using odct.Counters;

namespace odct.tests.Counters {
    [TestFixture]
    [Category("Integration")]
    public class InstanceHeaderTests {
        public byte[] testArray;
        public InstanceHeader ih;

        [TestFixtureSetUp]
        public void init() {
            ZapiContextStub zc = new ZapiContextStub(null, null);
            var gzipSource = new ZapiArchiveFileSource(zc) {
                Filename = "stats_archive_data_1447684431_20151116_093351.gz",
                Path = OdctSettingsTest.GetRelativeUnitTestDataDirectory("ontap73")
            };

            testArray = gzipSource.ReadAsBytes();
        }

        [Test]
        public void CanGetSystemCountersFromConfigCounter() {

            ih = new InstanceHeader(ref testArray, 132583);

            Assert.AreEqual(45251595752, ih.msecs_since_boot);
            Assert.AreEqual(676, ih.instance_size);
            Assert.AreEqual(0, ih.oid);
            Assert.AreEqual(1, ih.num_instances);
            Assert.AreEqual(917504, ih.instanceFlag.flagValue);

        }

        [Test]
        public void CanGetSystemCountersFromFreq1Counter() {

            // First system object record where config flag is false, so frequency_1 only is true:

            ih = new InstanceHeader(ref testArray, 180163);

            Assert.AreEqual(45251596752, ih.msecs_since_boot);
            Assert.AreEqual(256, ih.instance_size);
            Assert.AreEqual(0, ih.oid);
            Assert.AreEqual(1, ih.num_instances);
            Assert.AreEqual(262144, ih.instanceFlag.flagValue);

        }

        [Test]
        public void CanGetSystemCountersFromFreq2Counter() {

            ih = new InstanceHeader(ref testArray, 225315);

            Assert.AreEqual(45251600752, ih.msecs_since_boot);
            Assert.AreEqual(256, ih.instance_size);
            Assert.AreEqual(0, ih.oid);
            Assert.AreEqual(1, ih.num_instances);
            Assert.AreEqual(786432, ih.instanceFlag.flagValue);

        }

        [Test]
        public void CanGetSystemCountersFromFreq3Counter() {

            ih = new InstanceHeader(ref testArray, 844515);

            Assert.AreEqual(45251640759, ih.msecs_since_boot);
            Assert.AreEqual(256, ih.instance_size);
            Assert.AreEqual(0, ih.oid);
            Assert.AreEqual(1, ih.num_instances);
            Assert.AreEqual(1835008, ih.instanceFlag.flagValue);
        }

        [Test]
        public void CanGetSystemCountersFromFreq4Counter() {

            ih = new InstanceHeader(ref testArray, 1774659);

            Assert.AreEqual(45251700760, ih.msecs_since_boot);
            Assert.AreEqual(1120, ih.instance_size);
            Assert.AreEqual(0, ih.oid);
            Assert.AreEqual(1, ih.num_instances);
            Assert.AreEqual(3932160, ih.instanceFlag.flagValue);

        }

        [Test]
        public void CanGetVolumeCountersFromConfigCounter() {

            ih = new InstanceHeader(ref testArray, 171611);

            Assert.AreEqual(45251595759, ih.msecs_since_boot);
            Assert.AreEqual(6760, ih.instance_size);
            Assert.AreEqual(10, ih.oid);
            Assert.AreEqual(28, ih.num_instances);
            Assert.AreEqual(917504, ih.instanceFlag.flagValue);

        }

    }
}
