﻿using NUnit.Framework;
using odct;
using odct.Json;

namespace odct.tests {

    [TestFixture]
    [Category("Unit")]
    class JsonTests {

        [Test]
        public void canGetJsonTrue() {
            JsonTrue jt = new JsonTrue();
            Assert.AreEqual("true", jt.toString());
        }

        [Test]
        public void canGetJsonFalse() {
            JsonFalse jf = new JsonFalse();
            Assert.AreEqual("false", jf.toString());
        }

        [Test]
        public void canGetJsonNull() {
            JsonNull jn = new JsonNull();
            Assert.AreEqual("null", jn.toString());
        }

        [Test]
        public void canGetJsonString() {
            JsonString jt = new JsonString("test");
            Assert.AreEqual("\"test\"", jt.toString());
        }

        [Test]
        public void canGetJsonArray() {
            JsonArray ja = new JsonArray();
            ja.Add(new JsonTrue());
            ja.Add(new JsonFalse());
            ja.Add(new JsonNull());
            ja.Add(new JsonString("test"));
            Assert.AreEqual("[true,false,null,\"test\"]", ja.toString());
        }
    
        [Test]
        public void canGetJsonObject() {
            JsonObject jo = new JsonObject();
            jo.Add("test", new JsonFalse());
            Assert.AreEqual("{\n\"test\":false\n}", jo.toString());
        }

        [Test]
        public void canGetComplexJsonObject() {
            JsonArray ja = new JsonArray();
            ja.Add(new JsonTrue());
            ja.Add(new JsonFalse());
            ja.Add(new JsonNull());
            ja.Add(new JsonString("test"));
            JsonObject jo = new JsonObject();
            jo.Add("another", ja);
            Assert.AreEqual("{\n\"another\":[true,false,null,\"test\"]\n}", jo.toString());
        }

        [Test]
        public void canGetTrueFromJsonBoolStaticMethod() {
            Assert.AreEqual("true", JsonBool.getJsonBool(true).toString());
        }

        [Test]
        public void canGetFalseFromJsonBoolStaticMethod() {
            Assert.AreEqual("false", JsonBool.getJsonBool(false).toString());
        }

        [Test]
        public void canGetJsonTrueFromJsonBoolStaticMethod() {
            Assert.AreEqual(new JsonTrue().toString(), JsonBool.getJsonBool(true).toString());
        }

        [Test]
        public void canGetJsonFalseFromJsonBoolStaticMethod() {
            Assert.AreEqual(new JsonFalse().toString(), JsonBool.getJsonBool(false).toString());
        }

    }
}
