use Archive::Parser;
use Module::Load;
use Data::Dumper;
use JSON;
use strict;

my $FILTER = {};
my @COUNTER = undef;

my $filename = $ARGV[0];
my $frequency = $ARGV[1];
my $counter_file = $ARGV[2];

if(scalar @ARGV != 3) {
	print "Wrong number of arguments provided:";
	print "<FileName> <Frequency> <CounterFile>\n";
	exit(1);
}

open FH, $counter_file or die $@;;
my $counter_str = join("\n", <FH>);
close FH;

my $counter_hash = from_json( $counter_str, { utf8  => 1 } ) or die $@;
foreach my $obj(keys %{$counter_hash}) {
    @COUNTER = @{$counter_hash->{$obj}};
    my %hash = map { $_ => "1"} @COUNTER;
    $FILTER->{$obj} = \%hash;
}

$filename =~ s/\//\\/g;
        
eval {
    my $data = parse_file($filename);
    printData($data);
};
if($@) {
    print ($@);
}

sub parse_file {
    my $filename = shift;

    my $obj = Archive::Parser->spawn($filename);
    $obj->filterCounters($FILTER);
    
    my $data ={};
    $obj->parse($frequency, $data);

    return $data;
}

sub printData {
    my $printdata = shift;

    open FH, ">output.csv";
    print FH getHeader($printdata);

	foreach my $object (keys %{$FILTER}) {
		foreach my $time (keys %{$printdata->{$object}}) {
		   print FH get_formatted_data($time, $printdata->{$object}->{$time}, $object);
		}
	}
    close FH;
}

sub get_formatted_data {
    my ($time, $objectblock, $object) = @_;
    my $print_string = "$time";

	foreach my $instance (keys %{$objectblock}) {
		foreach my $cntr (@COUNTER) {
			if(exists $objectblock->{$instance}->{$cntr}) {
				$objectblock->{$instance}->{$cntr} =~ s/,/ /g;
				$print_string = $print_string.",".$objectblock->{$instance}->{$cntr};
			} else {
				$print_string = $print_string.", ";
			}
		}
	}

    return $print_string."\n";
}

sub getHeader {
    my $data = shift;
	my @header = ();
	foreach my $object (keys %{$FILTER}) {
        foreach my $time (keys %{$data->{$object}}) {
            foreach my $instance(keys %{$data->{$object}->{$time}}) {
                my @counters = @COUNTER;
                if(scalar @counters > 0) {
                    my @header_1 = map { $object.':'.$instance.':'.$_} @counters;
                    push @header, @header_1;
                }
            }
            last;
        }
	}
    unshift(@header, 'TIMESTAMP');
    my $joinedstr = join(',', @header);
    $joinedstr = $joinedstr."\n";
    return $joinedstr;
}