﻿namespace odct.Clusters {
    using System;
    using System.Collections.Generic;
    using odct.Settings;
    using odct.FilerConfig;

    public class ClusterConfiguration {
        private Cluster thisCluster;
        private static TimeSpan ALLOWED_SLEW = TimeSpan.FromDays(7);
        public IEnumerable<Section> Sections;
        public string[] SimpleZapiCommands = {
            "system-get-ontapi-version",
            "cluster-identity-get",
            "license-v2-list-info",         // LICENSES section
            "storage-shelf-info-get-iter",  // SHELF and SHELF_MODULE sections
            "system-get-version",           // SYSTEM section
            "system-node-get-iter",         // SYSTEM section
        };

        public ClusterConfiguration(Cluster thisCluster) {
            this.thisCluster = thisCluster;
        }

        public void Build(Credentials credentials, Messenger messenger) {

            messenger.sendInfo(string.Format("Cluster {0} ClusterConfiguration.Build() started.", this.thisCluster.clusterName));
            var sections = new List<Section>();

            bool clockGetClockIsSupported = true;
            if ("8.0".Equals(this.thisCluster.ontapMajorVersion)
                || "8.1".Equals(this.thisCluster.ontapMajorVersion)
                || "8.2".Equals(this.thisCluster.ontapMajorVersion))
                clockGetClockIsSupported = false;

            if (clockGetClockIsSupported) {
                var clock = new Clock(credentials, messenger);
                if (!TimeUtil.TimeSlewWithinRange(clock.FilerCurrentDateTime, DateTime.UtcNow, ALLOWED_SLEW)) {
                    throw new OperationCanceledException("Local system time is too far out of range.");
                }
                sections.Add(clock);
            }

            /* Removed due to QA feedback that these zapi's are unneeded.  If this is proven to be true
             * after odct web is complete, delete this section of code and the ClusterStorageAdapterGetAdapterInfo
             * and ClusterStorageAdapterGetAdapterList objects.
             * 
            var adapters = new ClusterStorageAdapterGetAdapterList(credentials, messenger);
            sections.Add(adapters);
            foreach (var nodeAdapterPair in adapters.NodeAdapterPairs) {
                sections.Add(new ClusterStorageAdapterGetAdapterInfo(credentials, messenger, nodeAdapterPair));
            }
            */

            var aggregates = new AggregatesAggrGetIter(credentials, messenger);
            sections.Add(aggregates);
            foreach (var aggregate in aggregates.AggregateNames) {
                sections.Add(new AggregatesGetAggregateOptions(credentials, messenger, aggregate));
            }

            var volumes = new ClusterVolumeGetIter(credentials, messenger);
            sections.Add(volumes);
            foreach (VolumeVServerPair volume in volumes.volumeVServerPairs) {
                sections.Add(new ClusterVolumeOptionsListInfo(credentials, messenger, volume));
            }
            foreach (var s in SimpleZapiCommands) {
                sections.Add(new SimpleZapiCommand(s, credentials, messenger));
            }

            ClusterOptions clusterOptions = new ClusterOptions(credentials, messenger);
            sections.Add(clusterOptions);

            this.Sections = sections;
        }
    }
}
