﻿namespace odct.Clusters {
    using System;
    using System.Collections.Generic;
    using System.Text.RegularExpressions;
    using NetApp.Manage;
    using odct.Settings;
    using odct.FilerConfig;

    public class ClusterNodeConfiguration {
        private ClusterNode thisNode;
        public IEnumerable<Section> Sections;
        public string[] SimpleZapiCommands = {
        };

        public ClusterNodeConfiguration(ClusterNode thisNode) {
            this.thisNode = thisNode;
        }

        public void Build(Credentials credentials, Messenger messenger) {

            messenger.sendDebug("ClusterNodeConfiguration.Build() started.");
            var sections = new List<Section>();

            sections.Add(new NodeStorageShelfEnvironmentListInfo(credentials, messenger, this.thisNode));

            // for ADAPTER section:
            sections.Add(new ClusterNodeNetInterfaceGetIter(credentials, messenger, thisNode));
            sections.Add(new ClusterNodeFcpAdapterGetIter(credentials, messenger, thisNode));

            // for DEVICE section:
            sections.Add(new ClusterNodeStorageDiskGetIter(credentials, messenger, thisNode));

            // for IOXM presence:
            string cliCommand = string.Format("system node run {0} sysconfig -a", thisNode.nodeName);
            sections.Add(new ClusterNodeCli(thisNode, cliCommand, credentials, messenger));

            // for adapters:
            cliCommand = string.Format("system node run {0} sysconfig -ac", thisNode.nodeName);
            sections.Add(new ClusterNodeCli(thisNode, cliCommand, credentials, messenger));

            // for PARTITION_DISK section:
            cliCommand = string.Format("system node run {0} sysconfig -r", thisNode.nodeName);
            sections.Add(new ClusterNodeCli(thisNode, cliCommand, credentials, messenger));

            // for OPTIONS sections
            sections.Add(new ClusterNodeOptionsListInfo(this.thisNode, credentials, messenger));

            // for SYSTEM section:
            sections.Add(new ClusterNodeSystemGetNodeInfoIter(credentials, messenger, thisNode));

            this.Sections = sections;
        }
    }
}
