namespace odct.Counters {
    using System;
    using System.IO;
    using System.IO.Compression;
    using System.Collections.Generic;
    using System.Text.RegularExpressions;
    using NetApp.Manage;

    public abstract class ArchiveFileSource {
        public string Filename { get; set; }
        public string Path { get; set; }
        public Stream Stream { get; set; }
        public abstract byte[] ReadAsBytes();

        // based on http://www.dotnetperls.com/decompress
        public static byte[] ReadGZippedByteArrayAsBytes(byte[] inputBytesBuffer) {
            const int readBufferSize = 4096;
            var readZipMemoryStream = new MemoryStream(inputBytesBuffer);
            var readZipStream = new GZipStream(readZipMemoryStream, CompressionMode.Decompress);
            var readChunkBuffer = new byte[readBufferSize];
            var outputMemoryStream = new MemoryStream();

            for (int readChunkLength = 1; readChunkLength > 0; ) {
                readChunkLength = readZipStream.Read(readChunkBuffer, 0, readBufferSize);
                outputMemoryStream.Write(readChunkBuffer, 0, readChunkLength);
            }

            byte[] returnArray = outputMemoryStream.ToArray();
            outputMemoryStream.Dispose();
            readZipStream.Dispose();
            return returnArray;
        }

        public static string GetFilerArchivePathZapi(ZapiContext zc) {

            NaServer nas = zc.GetServer();
            NaElement api = new NaElement("system-get-version");
            NaElement xo = nas.InvokeElem(api);
            string version = xo.GetChildContent("version");
            string systemVersion = Regex.Match(version, "\\d\\.\\d\\.\\d").Value;

            const string archivePath_7_3 = "/vol/vol0/etc/stats/archive";
            const string archivePath_8_1 = "/vol/vol0/etc/log/stats/archive";
            const string archivePath_8_2 = "/vol/vol0/etc/log/stats/archive";

            if (Regex.Match(systemVersion, "^7.3").Success) {
                return archivePath_7_3;
            } else if (Regex.Match(systemVersion, "^8.1").Success) {
                return archivePath_8_1;
            } else if (Regex.Match(systemVersion, "^8.2").Success) {
                return archivePath_8_2;
            } else {
                return null;
            }
        }
    }
}
