﻿namespace odct.Counters {
    using System;
    using System.Collections.Generic;
    using System.Text.RegularExpressions;
    using odct.Settings;

    public class ArchiveParserFactory {
        private string SystemSerialNo;
        private Messenger messenger;
        private ArchiveFileSource source;
        private string outputPath;
        private Dictionary<string, int> requestedFrequencyByDottedId;
        private bool writeXmlToPayload;
        private bool isClusterMode;
        private string appCountersRequested;

        public ArchiveParserFactory(string serial, Messenger m, ArchiveFileSource src, string targetPath, string appCountersRequested, bool writeXmlToPayload, bool isClusterMode) {
            this.SystemSerialNo = serial;
            this.messenger = m;
            this.source = src;
            this.outputPath = targetPath;
            this.appCountersRequested = appCountersRequested;
            this.writeXmlToPayload = writeXmlToPayload;
            this.isClusterMode = isClusterMode;
        }

        public ArchiveParser Build(string version) {
            if (Regex.Match(version, "^7.3").Success) {
                this.requestedFrequencyByDottedId = OdctSettings.AppToPerfCounters(this.appCountersRequested, "7.3");
                return new ArchiveParser_7_3(this.SystemSerialNo, this.messenger, this.source, this.outputPath, this.requestedFrequencyByDottedId, this.writeXmlToPayload);
            } else if (Regex.Match(version, "^8.1").Success && this.isClusterMode) {
                this.requestedFrequencyByDottedId = OdctSettings.AppToPerfCounters(this.appCountersRequested, "8.1", true);
                return new ArchiveParser_7_3(this.SystemSerialNo, this.messenger, this.source, this.outputPath, this.requestedFrequencyByDottedId, this.writeXmlToPayload);
            } else if (Regex.Match(version, "^8.1").Success && !this.isClusterMode) {
                this.requestedFrequencyByDottedId = OdctSettings.AppToPerfCounters(this.appCountersRequested, "8.1");
                return new ArchiveParser_8_1(this.SystemSerialNo, this.messenger, this.source, this.outputPath, this.requestedFrequencyByDottedId, this.writeXmlToPayload);
            } else if (Regex.Match(version, "^8.2").Success) {
                this.requestedFrequencyByDottedId = OdctSettings.AppToPerfCounters(this.appCountersRequested, "8.2");
                return new ArchiveParser_8_2(this.SystemSerialNo, this.messenger, this.source, this.outputPath, this.requestedFrequencyByDottedId, this.writeXmlToPayload);
            } else {
                return null;
            }
        }
    }
}
