using System;
using System.Collections.Generic;
using System.Text;

namespace odct.Counters {
    public class ArchiveParser_8_2 : ArchiveParser {
        public ArchiveParser_8_2(string serial, Messenger m, ArchiveFileSource s, string p, Dictionary<string, int> requestedFrequencyByDottedId, bool writeXmlToPayload)
            : base(serial, m, s, p, requestedFrequencyByDottedId, writeXmlToPayload) {

        }

        protected override void writeHeaderRows() {
            foreach (int oid in neededOids) {
                if (countersAreRequestedByOid[oid]) {
                    string objectName = counterDefinitions.GetObjectNameByOid(oid);
                    List<string> counterNames = getCounterNames(counterDefinitionListByOid[oid]);
                    writer[oid].Write("instanceFlag", objectName);
                    writer[oid].Write(",TIMESTAMP");
                    writer[oid].Write(",msecs_since_boot", objectName);
                    writer[oid].Write(",instance_num", objectName);
                    foreach (var name in counterNames) {
                        writer[oid].Write(",{0}:{1}", objectName, name);
                    }
                    writer[oid].WriteLine("");
                }
            }
        }

        protected override void writeDataRow(int thisRecordStart) {
            InstanceHeader ih = new InstanceHeader(ref allBytes, thisRecordStart);
            var offset = thisRecordStart + 40;
            if (countersAreRequestedByOid[(int) ih.oid]
                    && hasCounterValues(ref allBytes, counterDefinitionListByOid[ih.oid], ih.instanceFlag, offset, thisRecordStart)) {
                for (int i = 1; i <= ih.num_instances; i++) {
                    writer[ih.oid].Write("{0}", ih.instanceFlag.flagValue);
                    writer[ih.oid].Write(",{0}", getTimestampFromMsecsSinceBoot(ih.msecs_since_boot));
                    writer[ih.oid].Write(",{0}", ih.msecs_since_boot);
                    writer[ih.oid].Write(",{0}", i);
                    List<string> counterValues
                        = getCounterValues(ref allBytes, counterDefinitionListByOid[ih.oid], ih.instanceFlag, ref offset, thisRecordStart);
                    writer[ih.oid].Write(",{0}", getStringListAsCsv(counterValues));
                    writer[ih.oid].WriteLine("");
                }
                rowsWrittenByOid[(int) ih.oid]++;
            }
        }

    }
}
