using System;
using System.Collections.Generic;
using System.Text;
using odct.Clusters;
using odct.Settings;

namespace odct.Counters {
    public class Cluster81SourceGenerator : ArchiveSourceGenerator {

        private ClusterNode clusterNode;
        private List<Cluster81ArchiveFileSource> sources;
        private List<string> filenames;

        public Cluster81SourceGenerator(ClusterNode cn) {

            this.clusterNode = cn;
            this.sources = new List<Cluster81ArchiveFileSource>();

            // e.g., http://<node-ip>/spi/<node-name>/etc/log/stats/archive/
            const string httpString= "http://{0}/spi/{1}/etc/log/stats/archive/";

            // e.g., <A HREF=stats_archive_data_1468925789_20160719_105629.gz>
            const string regexString = "<A HREF=\"(?<filebase>stats_archive_data_\\d{10}_\\d{8}_\\d{6}.gz)\">";

            filenames = this.clusterNode.getArchiveBaseList(httpString, regexString);

            foreach (string fn in filenames) {
                Cluster81ArchiveFileSource source = new Cluster81ArchiveFileSource(this.clusterNode, fn);
                this.sources.Add(source);
            }

        }

        public IEnumerable<ArchiveFileSource> GetSources() {
            foreach (ArchiveFileSource source in sources) {
                yield return source;
            }
        }

        public int Count() {
            return sources.Count;
        }
    }
}
