﻿namespace odct.Counters {
    using System;
    using System.Collections.Generic;

    public class CounterDefinitionLookup {
        public string ObjectName { get; private set; }
        public int Oid { get; private set; }
        private Dictionary<int, CounterDefinition> counterDefinitionByCounterId;
        private Dictionary<string, CounterDefinition> counterDefinitionByCounterName;
        private ICounterDefinitionCollector counterDefinitionCollector;
        public bool hasHistograms { get; private set ;}

        public CounterDefinitionLookup(string metadataXml, string objectName) {
            this.ObjectName = objectName;
            this.counterDefinitionCollector = new CounterDefinitionCollector(metadataXml);
            this.counterDefinitionByCounterId = this.counterDefinitionCollector.getCounterDefinitionByCounterId(objectName);
            this.counterDefinitionByCounterName = new Dictionary<string, CounterDefinition>();

            foreach(var e in this.counterDefinitionByCounterId) {
                var counterDef = e.Value;
                if (counterDef.isHistogram()) { hasHistograms = true; }
                this.Oid = counterDef.oid; // only really need to do once
                this.counterDefinitionByCounterName.Add(counterDef.counterName, counterDef);
            }
        }

        public List<CounterDefinition> getCounters() {
            List<CounterDefinition> result = new List<CounterDefinition>();
            foreach (var kvp in counterDefinitionByCounterId) {
                result.Add(kvp.Value);
            }
            return result;
        }

        public string getCounterName(int counterId) {
            return counterDefinitionByCounterId[counterId].counterName;
        }

        public List<CounterDefinition> GetCounterDefinitionListSortedByCounterId() {
            var result = new List<CounterDefinition>();

            // since .net 2 has no Dictionary.Keys.ToArray()
            var keys = new int[this.counterDefinitionByCounterId.Keys.Count];
            this.counterDefinitionByCounterId.Keys.CopyTo(keys, 0);
            Array.Sort(keys);

            foreach (var k in keys) {
                var m = this.counterDefinitionByCounterId[k];
                result.Add(m);
            }

            return result;
        }

        public CounterDefinition GetCounterDefinitionByCounterId(int id) {
            return this.counterDefinitionByCounterId[id];
        }

        public CounterDefinition GetCounterDefinitionByCounterName(string name) {
            return this.counterDefinitionByCounterName[name];
        }
    }
}
