﻿namespace odct.Counters {
    using System;
    using System.Collections.Generic;

    public class CounterDefinitions {
        private const string GET_ALL_COUNTERS_SENTINEL = "";
        private Dictionary<string, CounterDefinitionLookup> counterDefinitionLookupByObjectName;
        private Dictionary<int, string> objectNameByOid;
        private Dictionary<string, int> oidByObjectName;

        public CounterDefinitions(string metadataXml, Dictionary<string, int> desiredFrequencyByDottedName) : 
            this(metadataXml, GetDictionaryKeys(desiredFrequencyByDottedName)) {
        }

        public static string[] GetDictionaryKeys(Dictionary<string, int> d) {
            // since .net 2 has no Dictionary.Keys.ToArray()
            var keys = new string[d.Keys.Count];
            d.Keys.CopyTo(keys, 0);
            return keys;
        }

        public CounterDefinitions(string metadataXml, string[] dottedNamedCounters) {
            var d = new Dictionary<string, List<string>>();

            foreach (var n in dottedNamedCounters) {
                if (n.Contains(".")) {
                    var s = n.Split('.');
                    addToOrCreateDictionaryListEntry(d, s[0], s[1]);
                } else {
                    addToOrCreateDictionaryListEntry(d, n, GET_ALL_COUNTERS_SENTINEL); 
                }
            }

            this.counterDefinitionLookupByObjectName = new Dictionary<string, CounterDefinitionLookup>();
            this.objectNameByOid = new Dictionary<int, string>();
            this.oidByObjectName = new Dictionary<string, int>();

            foreach (var e in d) {
                var objectName = e.Key;

                var bidi = new CounterDefinitionLookup(metadataXml, objectName);

                this.counterDefinitionLookupByObjectName.Add(objectName, bidi);
                var oid = bidi.Oid;
                if (!objectNameByOid.ContainsKey(oid)) this.objectNameByOid.Add(oid, objectName);
                if (!oidByObjectName.ContainsKey(objectName)) this.oidByObjectName.Add(objectName, oid);
            }
        }

        private static void addToOrCreateDictionaryListEntry(Dictionary<string, List<string>> d, string k, string v) {
            if (d.ContainsKey(k)) {
                d[k].Add(v);
            } else {
                d.Add(k, new List<string>() { v });
            }
        }

        private struct CounterName { public string objectName; public string counterName; }

        private CounterName? parse(string dottedName) {
            if (dottedName.Contains(".")) {
                var s = dottedName.Split('.');
                return new CounterName { objectName = s[0], counterName = s[1] };
            } else {
                return null;
            }
        }

        public CounterDefinition GetCounterDefinitionByDottedName(string dottedName) {
            var s = dottedName.Split('.');
            return this.counterDefinitionLookupByObjectName[s[0]].GetCounterDefinitionByCounterName(s[1]);
        }

        public long GetOidByObjectName(string objectName) {
            return this.oidByObjectName[objectName];
        }

        public string GetObjectNameByOid(int oid) {
            return this.objectNameByOid[oid];
        }

        public List<int> GetOidsToCollect() {
            return new List<int>(objectNameByOid.Keys);
        }

        public CounterDefinitionLookup GetCounterDefinitionLookup(string objectName) {
            return this.counterDefinitionLookupByObjectName[objectName];
        }

    }
}
