namespace odct.Counters {
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Net;
    using System.Text;
    using System.Xml;
    using NetApp.Manage;

    public class ZapiArchiveFileSource : ArchiveFileSource {

        public ZapiContext zapiContext;

        public ZapiArchiveFileSource(ZapiContext zc) {
            zapiContext = zc;
        }

        public override byte[] ReadAsBytes() {
            var gzipPath = System.IO.Path.Combine(
                this.Path,
                this.Filename);

            return ReadGZippedArchiveFileAsBytes(gzipPath);
        }

        public byte[] ReadGZippedArchiveFileAsBytes(string filename) {
            if (String.IsNullOrEmpty(this.Path)) {
                return ArchiveFileSource.ReadGZippedByteArrayAsBytes(zapiBatchReadAllBytes(filename, zapiContext));
            } else {
                return ArchiveFileSource.ReadGZippedByteArrayAsBytes(File.ReadAllBytes(filename)); // to allow older unit tests to run.
            }
        }

        public static byte[] zapiBatchReadAllBytes(string filename, ZapiContext zc) {
            NaServer nas = zc.GetServer();

            // get file size

            NaElement api = new NaElement("file-get-file-info");
            api.AddNewChild("path", filename);
            
            NaElement xo = nas.InvokeElem(api);

            const int defaultValue = 0;
            NaElement x1 = xo.GetChildByName("file-info");
            int length = x1.GetChildIntValue("file-size", defaultValue);

            byte[] resultArray = new byte[length];

            // build xml for POST

            const int maxChunkSize = 1000000;
            var remainingBytes = length;
            string xmlString = "";
            xmlString = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
            xmlString += "<netapp xmlns=\"http://www.netapp.com/filer/admin\" version=\"1.12\">\n";

            while (remainingBytes > 0) {

                var bytesThisIteration = Math.Min(remainingBytes, maxChunkSize);
                var offset = length - remainingBytes;

                xmlString += "<file-read-file>";
                xmlString += string.Format("<path>{0}</path>", filename);
                xmlString += string.Format("<length>{0}</length>", maxChunkSize);
                xmlString += string.Format("<offset>{0}</offset>", offset);
                xmlString += "</file-read-file>";

                remainingBytes -= maxChunkSize;
            }

            xmlString += "</netapp>\n";

            byte[] postContentAsBytes = Encoding.ASCII.GetBytes(xmlString);

           

            // get file in single POST
            string protocol = (zc.TransportType == NaServer.TRANSPORT_TYPE.HTTP) ? "http" : "https";
            string postUri = string.Format(
                "{0}://{1}/servlets/netapp.servlets.admin.XMLrequest_filer",
                protocol,
                zc.credentials.Server);
            HttpWebRequest request = (HttpWebRequest) WebRequest.Create(postUri);
            NetworkCredential nc = new NetworkCredential(zc.credentials.Username, zc.credentials.Password);
            request.Credentials = nc;
            request.ContentLength = postContentAsBytes.Length;
            request.ContentType = "application/x-www-form-urlencoded";
            request.Method = "POST";
            request.UserAgent = "Java/1.8.0_25";
            request.KeepAlive = true;
            request.Accept = "text/html, image/gif, image/jpeg, *; q=.2, */*; q=.2";

            Stream requestStream = request.GetRequestStream();
            requestStream.Write(postContentAsBytes, 0, postContentAsBytes.Length);
            requestStream.Close();

            WebResponse response = request.GetResponse();
            Stream responseStream = response.GetResponseStream();

            XmlReaderSettings xmlReaderSettings = new XmlReaderSettings() { ProhibitDtd = false, XmlResolver = null };

            int bytesReturned = 0;
            int thisoffset = 0;
/*
            string x;
            using (StreamReader reader = new StreamReader(responseStream, Encoding.UTF8))
            {
                x = reader.ReadToEnd();
            } 
*/
            using (XmlReader xmlReader = XmlReader.Create(responseStream, xmlReaderSettings)) {
                int doCount = 1;
                xmlReader.MoveToContent();
                xmlReader.ReadToFollowing("results");
                do {
                    xmlReader.ReadToDescendant("length");
                    int byteLength = xmlReader.ReadElementContentAsInt();
                    bytesReturned += byteLength;
                    xmlReader.Read();
                    var vark = xmlReader.ReadContentAsBinHex(resultArray, thisoffset, byteLength);
                    doCount++;
                    thisoffset += byteLength;
                } while (xmlReader.ReadToFollowing("results"));
            }

            response.Close();
            return resultArray;

        }
    
    }
}
