namespace odct.Counters {
    using System;
    using System.Collections.Generic;
    using NetApp.Manage;

    public class ZapiSourceGenerator : ArchiveSourceGenerator {

        private string sourcePath;
        private ZapiContext zapiContext;
        private string[] filenamesBackingStore = null;

        private string[] filenames { 
            get {
                if (this.filenamesBackingStore == null) {
                    this.filenamesBackingStore = this.GetFilenames();
                }

                return this.filenamesBackingStore;
        } }

        public ZapiSourceGenerator(string sourcePath, ZapiContext zapiContext) {
            this.sourcePath = sourcePath;
            this.zapiContext = zapiContext;
        }

        public string[] GetFilenames() {

            NaServer nas = zapiContext.GetServer();

            // In 7-mode we need to avoid trying to read the file now being written by ONTAP.
            NaElement api = new NaElement("clock-get-clock");
            NaElement xo = nas.InvokeElem(api);
            long filerLocalTime = xo.GetChildLongValue("local-time", 0);

            NaElement api2 = new NaElement("file-list-directory-iter-start");
            api2.AddNewChild("path", ArchiveFileSource.GetFilerArchivePathZapi(zapiContext));

            NaElement xo2 = nas.InvokeElem(api2);
            string tag = xo2.GetChildContent("tag");
            string records = xo2.GetChildContent("records");

            NaElement api3 = new NaElement("file-list-directory-iter-next");
            api3.AddNewChild("tag", tag);
            api3.AddNewChild("maximum", records);

            NaElement xo3 = nas.InvokeElem(api3);

            int fileCount = xo3.GetChildIntValue("records", 0);   // 0 is default value.

            NaElement filesElement = xo3.GetChildByName("files");
            List<NaElement> filesList = filesElement.GetChildren();

            for (int i = filesList.Count - 1; i >= 0; i--) {
                if ("directory".Equals(filesList[i].GetChildContent("file-type"))) {
                    filesList.RemoveAt(i);
                    continue;
                };
                if (filerLocalTime - filesList[i].GetChildLongValue("creation-timestamp", 0) < 3600) {
                    filesList.RemoveAt(i);
                }
            }

            string[] filesArray = new string[filesList.Count];
            string filepath = ArchiveFileSource.GetFilerArchivePathZapi(zapiContext);

            for (int i = 0; i < filesList.Count; i++) {
                filesArray[i] = string.Format("{0}/{1}", filepath, filesList[i].GetChildContent("name")
                );
            }

            Array.Sort(filesArray);
            Array.Reverse(filesArray);

            return filesArray;
        }

        public int Count() {
            return this.filenames.Length;
        }

        public IEnumerable<ArchiveFileSource> GetSources() {
            foreach (var f in filenames) {
                var src = new ZapiArchiveFileSource(this.zapiContext) {
                    Filename = f,
                    Path = "",
                };

                yield return src;
            }
        }
    }
}
