﻿using System;
using System.IO;
using System.Runtime.InteropServices;

namespace odct {
    public class EmbeddedUnmanagedDll {
        public string PathName { get; private set; }
        private IntPtr DllHandle { get; set; }

        public EmbeddedUnmanagedDll(byte [] resourceBytes, string filename) {
            this.SetPathNameAndRemoveExistingDll(filename);
            this.WriteAndLoadDll(resourceBytes);
        }

        private void SetPathNameAndRemoveExistingDll(string filename) {
            var tempFolder = new TempFolder();
            this.PathName = Path.Combine(tempFolder.PathName, filename);

            if (File.Exists(this.PathName)) {
                File.Delete(this.PathName);
            }
        }

        unsafe private void WriteAndLoadDll(byte[] bytes) {
            File.WriteAllBytes(this.PathName, bytes);

            this.DllHandle = LoadLibrary(this.PathName);

            if (this.DllHandle == IntPtr.Zero) {
                uint error = GetLastError();

                this.Cleanup();
            }
        }

        unsafe public bool Cleanup() {
            if (this.UnloadDll()) {
                return this.DeleteDll();
            }
            return false;
        }

        unsafe private bool UnloadDll() {
            IntPtr handle = this.DllHandle;

            if (handle == IntPtr.Zero) {
                handle = GetModuleHandle(this.PathName);
            }

            if (handle != IntPtr.Zero) {
                if (FreeLibrary(handle) == false) {
                    uint error = GetLastError();

                    if (UnmapViewOfFile(handle) == false) {
                        error = GetLastError();
                        return false;
                    }
                }
            }

            this.DllHandle = IntPtr.Zero;

            return true;
        }

        private bool DeleteDll() {
            if (this.PathName.Length > 0) {
                try {
                    File.Delete(this.PathName);
                    this.PathName = string.Empty;
                } catch (Exception) {
                    return false;
                }
            }

            return true;
        }

        [DllImport("kernel32", SetLastError = true, CharSet = CharSet.Ansi)]
        static extern IntPtr LoadLibrary([MarshalAs(UnmanagedType.LPStr)]string lpFileName);

        [DllImport("kernel32.dll", SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        static extern bool FreeLibrary(IntPtr hModule);

        [DllImport("kernel32.dll", SetLastError = true)]
        static extern bool UnmapViewOfFile(IntPtr lpBaseAddress);

        [DllImport("kernel32.dll")]
        static extern uint GetLastError();

        [DllImport("kernel32.dll", SetLastError = true, CharSet = CharSet.Auto)]
        public static extern IntPtr GetModuleHandle(string lpModuleName);
    }
}
