﻿namespace odct.FilerConfig {
    using System;
    using NetApp.Manage;

    public class PartnerNetResolve : Section {
        public string HostName { get; private set; }
        public string Address { get; private set; }

        public PartnerNetResolve(Credentials c, Messenger m, string hostname)
            : base(c, m) {
            this.HostName = hostname;
            this.FinishConstructor();
        }

        public override string GetZapiCommandString() {
            return "net-resolve";
        }

        public override string GetSectionTitle() {
            return string.Format("{0}-{1}", this.GetZapiCommandString(), this.HostName);
        }

        protected override NaElement Process(string specificZapiCommand = null) {
            var api = new NaElement(this.GetZapiCommandString());
            api.AddNewChild("host-name", this.HostName);
            NaElement xo = null;
            try {
                xo = this.credentials.naServer.InvokeElem(api);
                NaElement ipAddresses = xo.GetChildByName("ip-addresses");
                this.Address = ipAddresses.GetChildContent("ip-address");

            } catch (NaApiFailedException e) {
                if (e.ErrorNo == 13166) {
                    this.Address = null;
                    messenger.sendInfo(string.Format("NaApiFailedException encountered, ErrNo:{0} Reason:{1} .", e.ErrorNo, e.Message));
                    messenger.sendInfo(string.Format("DNS is not configured for {0}, so retrieval of its ip address failed.", HostName));
                    messenger.sendInfo(string.Format("Auto collection of partner {0} is not possible.", HostName));
                    messenger.sendInfo(string.Format("Manual collection of {0} is possible by providing explicit credentials.", HostName));
                    messenger.sendInfo("");
                }
            }
            return xo;
        }


    }
}
