﻿namespace odct.FilerConfig {
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Xml;
    using System.Xml.XPath;
    using NetApp.Manage;

    public abstract class Section {
        protected Credentials credentials;
        protected Messenger messenger;
        protected NaElement naElement { get; private set; }

        public Section(Credentials c, Messenger m) {
            this.credentials = c;
            this.messenger = m;
        }

        public abstract string GetZapiCommandString();

        public virtual string GetSectionTitle() {
            return this.GetZapiCommandString();
        }

        protected void FinishConstructor() {
            this.naElement = this.Process();
        }

        protected virtual NaElement Process(string specificZapiCommand = null) {
            var zapiCommand = specificZapiCommand ?? this.GetZapiCommandString();
            var api = new NaElement(zapiCommand);
            return this.SafeInvokeElement(api);
        }

        protected NaElement SafeInvokeElement(NaElement api) {
            NaElement xo = null;
            try {
                this.messenger.sendDebug(string.Format("Trying SafeInvokeElement for section {0}.", api.GetName()));
                xo = this.credentials.naServer.InvokeElem(api);
            } catch (Exception e) {
                this.messenger.sendWarning(string.Format("...Unexpected error in Section.Process(): {0}", e.Message));
            }
            this.messenger.sendDebug(string.Format("...SafeInvokeElement for section {0} returning.", api.GetName()));
            return xo;

        }

        public NaElement GetNaElement() {
            return naElement;
        }

    }
}
