﻿namespace odct.FilerConfig {
    using System;
    using NetApp.Manage;
    using odct.Settings;

    public class Status : Section {
        public DateTime FilerCurrentDateTime { get; private set; }
        public bool isHaPartner { get; private set; }

        public Status(Credentials c, Messenger m)
            : base(c, m) {
            this.isHaPartner = true;
            this.FinishConstructor();
        }

        public override string GetZapiCommandString() {
            return "cf-status";
        }

        protected override NaElement Process(string specificZapiCommand = null) {
            try {

                this.messenger.sendInfo("Status.Process started.");
                NaElement xo = base.Process();
                this.messenger.sendInfo(string.Format("Status.Process received \"{0}\" as xo.GetAttr(\"status\").", xo.GetAttr("status")));

                if (!"passed".Equals(xo.GetAttr("status"),StringComparison.OrdinalIgnoreCase)) {
                    this.messenger.sendDebug(string.Format("Status.Process processing \"status\" equals \"passed\"."));
                    this.messenger.sendDebug(string.Format("Status.Process setting isHaPartner to false."));
                    this.isHaPartner = false;
                    return null;
                };

                string currentMode = xo.GetChildContent("current-mode");
                this.messenger.sendInfo(string.Format("Status.Process received \"{0}\" as xo.GetChildContent(\"current-mode\").", xo.GetChildContent("current-mode")));

                if ("non_ha".Equals(currentMode, StringComparison.OrdinalIgnoreCase)) {
                    this.messenger.sendDebug(string.Format("Status.Process processing \"current-mode\" equals \"non_ha\"."));
                    this.messenger.sendDebug(string.Format("Status.Process setting isHaPartner to false."));
                    this.isHaPartner = false;
                    return null;
                }

                string partnerName = xo.GetChildContent("partner");
                if (string.IsNullOrEmpty(partnerName)) {
                    this.messenger.sendDebug(string.Format("Status.Process processing \"partner\" equals \"{0}\".", partnerName));
                    this.messenger.sendDebug(string.Format("Status.Process setting isHaPartner to false."));
                    this.isHaPartner = false;
                    return null;
                }

                return xo;

            } catch (Exception e) {
                this.isHaPartner = false;
                this.messenger.sendError(string.Format("cf-status returned error: {0}", e.Message));
                this.messenger.sendException(e);
                return null;
            }
        }
    }
}
