﻿using System;
using System.Collections.Generic;
using System.Text;
using odct.Settings;
using odct.View;

namespace odct {
    class ProgramVersionValidator {
        public const int NOT_EXPIRED = 0;
        public const int EXPIRING_SOON = 1;
        public const int EXPIRED = 2;

        private static DateTime ExpirationDate = new DateTime(2016, 12, 31);
        private static DateTime ExpireationWarningDate = ProgramVersionValidator.ExpirationDate.AddDays(-60);

        private Messenger Messenger = null;
        private bool ShowWarningMessagess { get; set; }

        public ProgramVersionValidator() : this (true) {
        }

        public ProgramVersionValidator(bool showWarnings) {
            this.ShowWarningMessagess = showWarnings;
        }

        public ProgramVersionValidator(Messenger messenger) {
            this.Messenger = messenger;
        }

        public bool CheckEverything() {
            bool allChecksPassed = this.CheckVersion();

            if (allChecksPassed) {
                allChecksPassed = this.CheckExpirationDate();
            }
            
            return (allChecksPassed);
        }

        public bool CheckVersion() {
            var version = new OdctVersion();
            int versionStatus = version.GetVersionStatus();

            if (versionStatus == OdctVersion.BELOW_MIN_VERSION) {
                string message = String.Format(odct.Properties.Resources.BELOW_MIN_VERSION_MESSAGE_TXT, new object[] { odct.Properties.Resources.ODCT_APP_NAME, version.Minimum, odct.Properties.Resources.ODCT_DOWNLOAD_URL });

                this.ShowError(odct.Properties.Resources.BELOW_MIN_VERSION_TITLE, message);
                return (false);
            } else if (versionStatus == OdctVersion.NEW_VERSION_AVAILABLE) {
                string message = String.Format(odct.Properties.Resources.NEW_VERSION_AVAILABLE_MESSAGE_TXT, new object[] { version.Latest, odct.Properties.Resources.ODCT_APP_NAME, odct.Properties.Resources.ODCT_DOWNLOAD_URL });

                this.ShowWarning(odct.Properties.Resources.NEW_VERSION_AVAILABLE_TITLE, message);
            }

            return true;
        }

        public bool CheckExpirationDate() {
            int expirationStatus = this.GetExpirationStatus();

            if (expirationStatus == ProgramVersionValidator.EXPIRED) {
                string title = String.Format(odct.Properties.Resources.ODCT_EXPIRED_TITLE, odct.Properties.Resources.ODCT_APP_NAME);
                string message = String.Format(odct.Properties.Resources.ODCT_EXPIRED_MESSAGE_TXT, new object[] { odct.Properties.Resources.ODCT_APP_NAME, ProgramVersionValidator.ExpirationDate.ToLongDateString(), odct.Properties.Resources.ODCT_DOWNLOAD_URL });

                this.ShowError(title, message);
                return (false);
            } else if (expirationStatus == ProgramVersionValidator.EXPIRING_SOON) {
                string title = String.Format(odct.Properties.Resources.ODCT_EXPIRING_SOON_TITLE, odct.Properties.Resources.ODCT_APP_NAME);
                string message = String.Format(odct.Properties.Resources.ODCT_EXPIRING_SOON_MESSAGE_TXT, new object[] { odct.Properties.Resources.ODCT_APP_NAME, ProgramVersionValidator.ExpirationDate.ToLongDateString(), odct.Properties.Resources.ODCT_DOWNLOAD_URL });

                this.ShowWarning(title, message);
            }

            return true;
        }

        private int GetExpirationStatus() {
            var now = DateTime.UtcNow;
            int status = ProgramVersionValidator.NOT_EXPIRED;

            if (now >= ProgramVersionValidator.ExpirationDate) {
                status = ProgramVersionValidator.EXPIRED;
            } else if (now >= ProgramVersionValidator.ExpireationWarningDate) {
                status = ProgramVersionValidator.EXPIRING_SOON;
            }

            return status;
        }

        private void ShowWarning(string title, string message) {
            if (this.ShowWarningMessagess) {
                if (this.Messenger != null) {
                    this.Messenger.sendWarning(message);
                } else {
                    HtmlMessageBox.Show(null, title, message);
                }
            }
        }

        private void ShowError(string title, string message) {
            if (this.Messenger != null) {
                this.Messenger.sendError(message);
            } else {
                HtmlMessageBox.Show(null, title, message);
            }
        }
    }
}
