﻿namespace odct.Settings {
    using System;
    using System.Collections.Generic;

    public class OdctSettings : SettingsFile, ICollectorOptions {
        // TODO: ControllerConnection to be moved to different configuration file to accomodate many hosts+creds
        public const string FilerSection = "ControllerConnection";
        public const string Hosts = "Hosts";
        public const int CollectAllPerformanceFiles = -1;

        public const string LocalMachineSection = "LocalMachine";
        public const string TargetPath = "OutputTargetPath";
        public const string DefaultTargetPath = @".\odcttest\dircopytest\dir1";
        public const string ParseTargetPath = @".\odcttest\parsetest\dir1";

        public const int LOGLEVEL_ERROR = 0;
        public const int LOGLEVEL_WARNING = 1;
        public const int LOGLEVEL_INFO = 2;
        public const int LOGLEVEL_DEBUG = 3;
        public const string LogLevelKey = "LogLevel";
        public const int DefaultLogLevel = LOGLEVEL_INFO;

        public const string DebugSection = "Debug";

        // TODO: consider moving logic into own class
        public const string CollectOptionsSection = "Collection";
        public const string AutoCollectHaPartners = "AutoCollectHaPartners";
        public const bool AutoCollectHaPartnersDefault = true;

        public const string DeleteCcmaCopies = "DeleteCcmaCopies";
        public const bool DeleteCcmaCopiesDefault = true;

        public const string PredefinedApplicationKey = "TargetApp";
        public const string DefaultPredefinedApplication = "trt";
        public const string ExplictCountersKey = "Counters";
        public const string PerformanceFilesCollectionLimitKey = "PerformanceFilesCollectionLimit";
        public const int ConfigCounterFrequency = 0;
        public const int DefaultCounterFrequency = 4;
        public const string DenyHttpKey = "DoNotAllowHTTP";
        public const bool DenyHttpDefault = false;

        public static Dictionary<string, int> AppToPerfCounters(string app, string version = "", bool isClusterMode = false) {

            Dictionary<string, int> trtDictionary = new Dictionary<string, int>()  { 
                    { "system.system_model", ConfigCounterFrequency },
                    { "system.ontap_version", DefaultCounterFrequency },
                    { "system.serial_no", DefaultCounterFrequency },
                    { "system.system_id", DefaultCounterFrequency },
                    { "system.hostname", DefaultCounterFrequency },
                    { "system.nfs_ops", DefaultCounterFrequency },
                    { "system.cifs_ops", DefaultCounterFrequency }, 
                    { "system.http_ops", DefaultCounterFrequency }, 
                    { "system.fcp_ops", DefaultCounterFrequency }, 
                    { "system.iscsi_ops", DefaultCounterFrequency },
                    { "system.sys_read_latency_hist", DefaultCounterFrequency },
                    { "system.sys_write_latency_hist", DefaultCounterFrequency },
                    { "system.sys_latency_hist", DefaultCounterFrequency },
                    { "system.net_data_recv", DefaultCounterFrequency },
                    { "system.net_data_sent", DefaultCounterFrequency },
                    { "system.disk_data_read", DefaultCounterFrequency },
                    { "system.disk_data_written", DefaultCounterFrequency },
                    { "system.cpu_busy", DefaultCounterFrequency },
                    { "system.cpu_elapsed_time", DefaultCounterFrequency },
                    { "system.avg_processor_busy", DefaultCounterFrequency },
                    { "system.cpu_elapsed_time1", DefaultCounterFrequency },
                    { "system.total_processor_busy", DefaultCounterFrequency },
                    { "system.cpu_elapsed_time2", DefaultCounterFrequency },
                    { "system.num_processors", DefaultCounterFrequency },
                    { "volume.parent_aggr", DefaultCounterFrequency },
                    { "volume.avg_latency", DefaultCounterFrequency },
                    { "volume.total_ops", DefaultCounterFrequency },
                    { "volume.read_data", DefaultCounterFrequency },
                    { "volume.read_latency", DefaultCounterFrequency },
                    { "volume.read_ops", DefaultCounterFrequency },
                    { "volume.write_data", DefaultCounterFrequency },
                    { "volume.write_latency", DefaultCounterFrequency },
                    { "volume.write_ops", DefaultCounterFrequency },
                    { "volume.other_latency", DefaultCounterFrequency },
                    { "volume.other_ops", DefaultCounterFrequency }};

            if ("trt".Equals(app) && (String.IsNullOrEmpty(version) || "7.3".Equals(version))
            ) {
                trtDictionary.Add("cifs.cifs_read_latency_hist", DefaultCounterFrequency);
                trtDictionary.Add("cifs.cifs_write_latency_hist", DefaultCounterFrequency);
                trtDictionary.Add("fcp.fcp_read_data", DefaultCounterFrequency);
                trtDictionary.Add("fcp.fcp_read_latency", DefaultCounterFrequency);
                trtDictionary.Add("fcp.fcp_read_ops", DefaultCounterFrequency);
                trtDictionary.Add("fcp.fcp_write_data", DefaultCounterFrequency);
                trtDictionary.Add("fcp.fcp_write_latency", DefaultCounterFrequency);
                trtDictionary.Add("fcp.fcp_write_ops", DefaultCounterFrequency);
                trtDictionary.Add("iscsi.iscsi_read_data", DefaultCounterFrequency);
                trtDictionary.Add("iscsi.iscsi_read_latency", DefaultCounterFrequency);
                trtDictionary.Add("iscsi.iscsi_read_ops", DefaultCounterFrequency);
                trtDictionary.Add("iscsi.iscsi_write_data", DefaultCounterFrequency);
                trtDictionary.Add("iscsi.iscsi_write_latency", DefaultCounterFrequency);
                trtDictionary.Add("iscsi.iscsi_write_ops", DefaultCounterFrequency);
                trtDictionary.Add("nfsv3.nfsv3_read_ops", DefaultCounterFrequency);
                trtDictionary.Add("nfsv3.nfsv3_write_ops", DefaultCounterFrequency);
                trtDictionary.Add("nfsv4.nfsv4_read_ops", DefaultCounterFrequency);
                trtDictionary.Add("nfsv4.nfsv4_write_ops", DefaultCounterFrequency);
                return trtDictionary;
            };

            if ("trt".Equals(app) && ("8.1".Equals(version) && isClusterMode)
            )
            {
                return trtDictionary;
            };

            if ("trt".Equals(app) && ("8.1".Equals(version) && !isClusterMode)
            )
            {
                trtDictionary.Add("volume.instance_name", DefaultCounterFrequency);
                trtDictionary.Add("cifs.cifs_read_latency_hist", DefaultCounterFrequency);
                trtDictionary.Add("cifs.cifs_write_latency_hist", DefaultCounterFrequency);
                trtDictionary.Add("fcp.fcp_read_data", DefaultCounterFrequency);
                trtDictionary.Add("fcp.fcp_read_latency", DefaultCounterFrequency);
                trtDictionary.Add("fcp.fcp_read_ops", DefaultCounterFrequency);
                trtDictionary.Add("fcp.fcp_write_data", DefaultCounterFrequency);
                trtDictionary.Add("fcp.fcp_write_latency", DefaultCounterFrequency);
                trtDictionary.Add("fcp.fcp_write_ops", DefaultCounterFrequency);
                trtDictionary.Add("iscsi.iscsi_read_data", DefaultCounterFrequency);
                trtDictionary.Add("iscsi.iscsi_read_latency", DefaultCounterFrequency);
                trtDictionary.Add("iscsi.iscsi_read_ops", DefaultCounterFrequency);
                trtDictionary.Add("iscsi.iscsi_write_data", DefaultCounterFrequency);
                trtDictionary.Add("iscsi.iscsi_write_latency", DefaultCounterFrequency);
                trtDictionary.Add("iscsi.iscsi_write_ops", DefaultCounterFrequency);
            };

            if ("trt".Equals(app) && "8.2".Equals(version))
            {
                trtDictionary.Add("system.hdd_data_read", DefaultCounterFrequency);
                trtDictionary.Add("system.hdd_data_written", DefaultCounterFrequency);
                trtDictionary.Add("volume.instance_name", DefaultCounterFrequency);
                trtDictionary.Add("volume.vserver_name", DefaultCounterFrequency);
                trtDictionary.Add("volume.vserver_uuid", DefaultCounterFrequency);
                trtDictionary.Add("fcp.fcp_read_data", DefaultCounterFrequency);
                trtDictionary.Add("fcp.fcp_read_ops", DefaultCounterFrequency);
                trtDictionary.Add("fcp.fcp_write_data", DefaultCounterFrequency);
                trtDictionary.Add("fcp.fcp_write_ops", DefaultCounterFrequency);
                trtDictionary.Add("iscsi.iscsi_read_data", DefaultCounterFrequency);
                trtDictionary.Add("iscsi.iscsi_read_ops", DefaultCounterFrequency);
                trtDictionary.Add("iscsi.iscsi_write_data", DefaultCounterFrequency);
                trtDictionary.Add("iscsi.iscsi_write_ops", DefaultCounterFrequency);
                trtDictionary.Add("volume.cifs_protocol_read_latency", DefaultCounterFrequency);
                trtDictionary.Add("volume.cifs_protocol_write_latency", DefaultCounterFrequency);
                trtDictionary.Add("volume.fcp_protocol_read_latency", DefaultCounterFrequency);
                trtDictionary.Add("volume.fcp_protocol_write_latency", DefaultCounterFrequency);
                trtDictionary.Add("volume.iscsi_protocol_read_latency", DefaultCounterFrequency);
                trtDictionary.Add("volume.iscsi_protocol_write_latency", DefaultCounterFrequency);
                trtDictionary.Add("volume.nfs_protocol_read_latency", DefaultCounterFrequency);
                trtDictionary.Add("volume.nfs_protocol_write_latency", DefaultCounterFrequency);
                return trtDictionary;
            };

            if ("synergy".Equals(app)) {
                return new Dictionary<string, int>();
            };

            return new Dictionary<string, int>();

        }


        public const string InterfaceSection = "Interface";

        private const string DefaultFilename = "odct-settings.xml";

        private static OdctSettings instance = null;

        private static object objLock = new object();

        public static void InitializeInstance(string filename) {
            lock (objLock) {
                if (instance != null) {
                    throw new System.InvalidOperationException("OdctSettings already initialized");
                }

                instance = new OdctSettings(filename);
            }
        }

        public static bool IsInit() {
            return instance != null;
        }

        public static OdctSettings Instance {
            get {
                lock (objLock) {
                    if (instance == null) {
                        instance = new odct.Settings.OdctSettings(null);
                    }

                    return instance;
                }
            }
        }

        private OdctSettings(string configuredFilename)
            : base(configuredFilename ?? OdctSettings.DefaultFilename, true) {
        }

        public bool GetAutoCollectHaPartners() {
            return this.GetBooleanValue(
                odct.Settings.OdctSettings.CollectOptionsSection,
                odct.Settings.OdctSettings.AutoCollectHaPartners,
                odct.Settings.OdctSettings.AutoCollectHaPartnersDefault);
        }

        public bool GetDeleteCcmaCopiesAfterProcessing() {
            return this.GetBooleanValue(
                odct.Settings.OdctSettings.CollectOptionsSection,
                odct.Settings.OdctSettings.DeleteCcmaCopies,
                odct.Settings.OdctSettings.DeleteCcmaCopiesDefault);
        }

        public Dictionary<string, int> GetCounterDefinitions() {
            var app = this.GetValue(
                odct.Settings.OdctSettings.CollectOptionsSection,
                odct.Settings.OdctSettings.PredefinedApplicationKey,
                odct.Settings.OdctSettings.DefaultPredefinedApplication);

            var explicitCountersString = this.GetValue(
                odct.Settings.OdctSettings.CollectOptionsSection,
                odct.Settings.OdctSettings.ExplictCountersKey,
                string.Empty);

            return GetCounterDefinitions(app, explicitCountersString);
        }

        public string getAppCountersRequested() {
            return "trt";
        }

        public static Dictionary<string, int> GetCounterDefinitions(string app, string explicitCountersString) {
            var explicitCounters = explicitCountersString.Split(',');
            var appCounters = AppToPerfCounters(app);
            var counters = new Dictionary<string, int>(appCounters);

            foreach (var cs in explicitCounters) {
                var c = parseCounter(cs);

                if (!string.IsNullOrEmpty(c.name)) {
                    if (counters.ContainsKey(c.name)) {
                        counters[c.name] = c.freq == ConfigCounterFrequency
                            ? counters[c.name]
                            : Math.Min(counters[c.name], c.freq);
                    } else {
                        counters[c.name] = c.freq;
                    }
                }
            }

            return counters;
        }

        private struct counter {
            public string name;
            public int freq;
            public counter(string name, int freq) {
                this.name = name;
                this.freq = freq;
            }
        }

        private static counter parseCounter(string c) {
            var components = c.Trim().Split(':');
            var name = components[0];
            var freq = components.Length > 1
                ? int.Parse(components[1])
                : odct.Settings.OdctSettings.DefaultCounterFrequency;

            return new counter(name, freq);
        }

        public ICollection<Credentials> GetHostConnections() {
            var hosts = OdctSettings.Instance.GetValue(
                odct.Settings.OdctSettings.FilerSection,
                odct.Settings.OdctSettings.Hosts,
                string.Empty)
                .Split(';');

            // TODO: eventually need to use regex or \t and \n delimit since ; and , are allowable in password

            var creds = new List<Credentials>();

            foreach (var h in hosts) {
                if (!string.IsNullOrEmpty(h.Trim())) {
                    creds.Add(new Credentials(h));
                }
            }

            return creds;
        }

        public bool DenyHttp() {
            return this.GetBooleanValue(FilerSection, DenyHttpKey, DenyHttpDefault);
        }

        public int GetPerformanceFilesCollectionLimit() {
            int work = this.GetIntValue(CollectOptionsSection, PerformanceFilesCollectionLimitKey, CollectAllPerformanceFiles);
            if (work == OdctSettings.CollectAllPerformanceFiles) {
                work = Int32.MaxValue;
            }
            return work;
        }

        public int GetLogLevel() {
            return this.GetIntValue(LocalMachineSection, LogLevelKey, DefaultLogLevel);
        }

        private string GetSettingDefaultValue(string key) {
            return string.Empty;
        }

        public string GetTargetPath() {
            return this.GetValue(
                    OdctSettings.LocalMachineSection,
                    OdctSettings.TargetPath,
                    OdctSettings.DefaultTargetPath);
        }
    }
}