﻿using System;
using System.IO;
using odct.Settings;

namespace odct {
    public class TempFolder {
        private static string folderPath = string.Empty;
        private delegate bool IterateAction(string folderName);

        public string PathName {
            get {
                return TempFolder.folderPath;
            }

            private set {
                TempFolder.folderPath = value;
            }
        }

        public TempFolder() {
            if (this.PathName.Length <= 0) {
                this.PathName = this.FindSuitableTempFolderName();
                Directory.CreateDirectory(this.PathName);
                if (!OdctSettings.Instance.GetDeleteCcmaCopiesAfterProcessing()) {
                    TempFolder.setFolderLockTrue(this.PathName, "Do not delete due to user option.");
                }
            }
        }

        public void Cleanup() {
            TempFolder.DeleteFolderIfItExistsAndIsUnlocked(this.PathName);
            this.PathName = string.Empty;
            this.DeleteOldTempFolders();
        }

        private string FindSuitableTempFolderName() {
            return this.IterateThroughTempFolderNames(TempFolder.CheckFolderNameAndDeleteIfItExists);
        }

        private void DeleteOldTempFolders() {
            this.IterateThroughTempFolderNames(TempFolder.DeleteFolderIfItExistsAndIsUnlocked);
        }

        private string IterateThroughTempFolderNames(IterateAction action) {
            string basePath = Path.Combine(Path.GetTempPath(), "NetApp.ODCT");

            if (action(basePath)) {
                return basePath;
            }

            for (int x = 0; x <= 99; x++) {
                string folderName = string.Format("{0}.{1}", basePath, x);

                if (action(folderName)) {
                    return folderName;
                }
            }

            return basePath;
        }

        public static bool DeleteFolderIfItExistsAndIsUnlocked(string folderName) {
            if (Directory.Exists(folderName)
                && !TempFolder.isFolderLocked(folderName)) {
                try {
                    Directory.Delete(folderName, true);  // true forces deletion of all files and subdirectories.
                    return true;
                } catch (Exception) {
                }
            }
            return false;
        }

        private static bool CheckFolderNameAndDeleteIfItExists(string folderName) {
            TempFolder.DeleteFolderIfItExistsAndIsUnlocked(folderName);

            if (Directory.Exists(folderName)) {
                return false;
            }

            if (File.Exists(folderName)) {
                return false;
            }

            return true;
        }

        private static bool isFolderLocked(string folderPath) {
            // Processes may indicate that a temporary folder should not be deleted by creating 
            // a file named lockfile.txt within it.  The directory may still be deleted manually, 
            // or may be flagged for automatic deletion by deleting this file.
            string lockfile = Path.Combine(folderPath, "lockfile.txt");
            return File.Exists(lockfile);
        }

        public static void setFolderLockTrue(string folderPath, string reason) {
            string lockfile = Path.Combine(folderPath, "lockfile.txt");
            string reasonText = string.Format("{0}\n", reason);
            if (File.Exists(lockfile)) {
                File.AppendAllText(lockfile, reasonText);
            } else {
                File.WriteAllText(lockfile, reasonText);
            }
        }

    }
}
