using System;
using System.Collections.Generic;

namespace odct {
    public class VerifyResults {

        public bool IsVerified { get; set;}
        public bool IsClusterMode { get; set;}
        public bool IsAuthenticated { get; set; }
        public bool IsHttps { get; set; }
        public bool IsFound { get; set; }
        public bool IsVersion8_0 { get; set; }
        public string FilerSystemId { get; set;}
        public string PartnerSystemId { get; set;}
        public string FilerSerialNumber { get; set;}
        public Exception VerifyException { get; set; }
        public string resultDescription { get; private set; }

        public VerifyResults() {
            IsClusterMode = false;
            IsVerified = false;
            IsAuthenticated = false;
            IsHttps = false;
            IsFound = false;
            IsVersion8_0 = false;
            FilerSystemId = "";
            FilerSerialNumber = "";
            PartnerSystemId = "";
            VerifyException = null;
        }

        public void setResultDescription() {
            resultDescription = "Credentials were successfully verified.";

            if (!IsAuthenticated) {
                resultDescription = "Connection refused with the credentials provided.";
            };

            if (!IsFound) {
                resultDescription = "Unable to connect at the address provided.";
            };

            if (IsClusterMode) {
                resultDescription = "Cluster identified. " + resultDescription;
            };

            if (IsVersion8_0) {
                resultDescription = "8.0 filer identified.  ODCT does not support 8.0 data collections.";
            };

        }

        public void logResults(Messenger m, string caller, string server) {
            m.sendInfo(string.Format("{0} received IsVerified:{1} from Verify on server: {2}. ", 
                caller, this.IsVerified, server));
            m.sendInfo(string.Format("{0} received IsClusterMode:{1} from Verify on server: {2}. ", 
                caller, this.IsClusterMode, server));
            m.sendInfo(string.Format("{0} received IsAuthenticated:{1} from Verify on server: {2}. ",
                caller, this.IsAuthenticated, server));
            m.sendInfo(string.Format("{0} received IsHttps:{1} from Verify on server: {2}. ",
                caller, this.IsHttps, server));
            m.sendInfo(string.Format("{0} received IsFound:{1} from Verify on server: {2}. ",
                caller, this.IsFound, server));            
            m.sendInfo(string.Format("{0} received IsVersion8_0:{1} from Verify on server: {2}. ", 
                caller, this.IsVersion8_0, server));
            m.sendInfo(string.Format("{0} received FilerSystemId:{1} from Verify on server: {2}. ",
                caller, this.FilerSystemId, server));
            m.sendInfo(string.Format("{0} received FilerSerialNumber:{1} from Verify on server: {2}. ",
                caller, this.FilerSerialNumber, server));
            m.sendInfo(string.Format("{0} received PartnerSystemId:{1} from Verify on server: {2}. ",
                caller, this.PartnerSystemId, server));
            if (this.VerifyException != null) {
                m.sendInfo(string.Format("{0} received an Exception from Verify on server: {1}. ",
                    caller, server));
                m.sendException(VerifyException);
            }
            m.sendInfo("");
        }
    }
}
