﻿using System;
using System.Text.RegularExpressions;
using System.Windows.Forms;

namespace odct.View {
    public partial class HtmlMessageBox : Form {
        private const string blankMessage = "about:blank";
        private const string bodyStyle = "font-family: arial";
        private string htmlMessage = HtmlMessageBox.blankMessage;
        private string windowTitle = string.Empty;
        private bool messageIsSet = false;

        private HtmlMessageBox(string title, string message) {
            this.InitializeComponent();
            
            this.windowTitle = title;
            this.htmlMessage = this.ConvertMessageToHtmlIfNecessary(message);
        }

        public static DialogResult Show(Form parent, string title, string message) {
            HtmlMessageBox messageBox = new HtmlMessageBox(title, message);

            return messageBox.ShowDialog(parent);
        }

        private void HtmlMessageBox_Load(object sender, System.EventArgs e) {
            this.ApplyTitleAndMessage();
            this.Browser.Document.BackColor = this.MessageBoxLayout.BackColor;
            this.AdjustWindowHeightToAccommodateMessage();

        }

        private string ConvertMessageToHtmlIfNecessary(string message) {
            if (!message.ToLower().Contains("<html>")) {
                string newMessage = this.ConvertMessageNewlinesToBreakTags(message);

                return string.Format("<html><body><p>{0}</p></body></html>", this.ConvertMessageUrlsToAnchorTags(newMessage));
            }

            return message;
        }

        private string ConvertMessageNewlinesToBreakTags(string message) {
            string newMessage = message;
            int startIndex = 0;
            int newlineIndex = newMessage.IndexOf(Environment.NewLine, startIndex);

            while (newlineIndex > 0) {
                string leftString = newMessage.Substring(0, newlineIndex);
                string rightString = newMessage.Substring(newlineIndex);

                newMessage = string.Format("{0}<br>{1}", leftString, rightString);
                startIndex = newlineIndex + Environment.NewLine.Length + 4;
                newlineIndex = newMessage.IndexOf(Environment.NewLine, startIndex);
            }

            return newMessage;
        }

        private string ConvertMessageUrlsToAnchorTags(string message) {
            var newMessage = message;
            int startIndex = 0;
            var regex = new Regex("(http|ftp|https)://([\\w_-]+(?:(?:\\.[\\w_-]+)+))([\\w.,@?^=%&:/~+#-]*[\\w@?^=%&/~+#-])?");
            var match = regex.Match(newMessage, startIndex);

            while (match.Success) {
                string leftString = newMessage.Substring(0, match.Index);
                string rightString = newMessage.Substring(match.Index + match.Length);
                string anchorTag = string.Format("<a href=\"{0}\">{0}</a>", newMessage.Substring(match.Index, match.Length));

                newMessage = string.Concat(leftString, anchorTag, rightString);
                startIndex = match.Index + anchorTag.Length;
                match = regex.Match(newMessage, startIndex);
            }

            return newMessage;
        }

        private void ApplyTitleAndMessage() {
            this.Text = this.windowTitle;

            // Don't set DocumentText directly - this is more reliable...
            this.Browser.Navigate(HtmlMessageBox.blankMessage);
            this.Browser.Document.Write(this.htmlMessage);

            this.messageIsSet = true;
        }

        private void AdjustWindowHeightToAccommodateMessage() {
            if (this.Browser.Document.Body != null) {
                this.Browser.Document.Body.Style = HtmlMessageBox.bodyStyle;

                int scrollHeight = this.Browser.Document.Body.ScrollRectangle.Height;
                int clientHeight = this.Browser.Document.Body.ClientRectangle.Height;

                this.Height += (scrollHeight - clientHeight);
            }
        }

        private void Browser_Navigating(object sender, WebBrowserNavigatingEventArgs e) {
            if (this.messageIsSet) {
                e.Cancel = true;
                this.LaunchExternalBrowser(e.Url.AbsoluteUri);
            }
        }

        private void LaunchExternalBrowser(string url) {
            System.Diagnostics.Process.Start(url);
        }
    }
}
