namespace odct.View {
    using System;
    using System.ComponentModel;
    using System.Collections.Generic;
    using System.Globalization;
    using System.Diagnostics;
    using odct.Settings;

    public class MessageProgress : Messenger {
        ProgressWindow pw;
        BackgroundWorker bw;
        private Action<object> work;
        private int logLevel;

        public MessageProgress() {
            this.logLevel = OdctSettings.Instance.GetLogLevel();
            this.bw = new BackgroundWorker();
            this.bw.WorkerReportsProgress = true;
            this.bw.WorkerSupportsCancellation = true;
            this.bw.DoWork += new DoWorkEventHandler(bw_DoWork);
            this.bw.ProgressChanged += new ProgressChangedEventHandler(bw_ProgressChanged);
            this.bw.RunWorkerCompleted += new RunWorkerCompletedEventHandler(bw_RunWorkerCompleted);
        }

        public void SetWork(Action<object> work) {
            this.work = work;
        }

        public void InitAndShow(int initialMax, Action<object> cancel) {
            this.pw = new ProgressWindow(cancel);
            this.pw.ProgressReset(initialMax);
            this.bw.RunWorkerAsync();
            this.pw.SetProgressLabelStep("Collection started...");
            this.pw.ShowDialog();
        }

        private void bw_DoWork(object sender, DoWorkEventArgs e) {
            var worker = sender as BackgroundWorker;
            this.work.Invoke(null);
        }

        private void bw_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e) {
            if (e.Cancelled) {
                this.pw.SetProgressLabelStep("Canceled!");
            } else if (!(e.Error == null)) {
                this.pw.SetProgressLabelStep("Error: " + e.Error.Message);
            } else {
                this.pw.SetProgressLabelStep("Done!");
                this.pw.SetButtonLabel("E&xit");
            }
        }

        private void bw_ProgressChanged(object sender, ProgressChangedEventArgs e) {
            var o = (updateObject) e.UserState;

            TextInfo usInfo = new CultureInfo("en-US", false).TextInfo;

            string prefix = "";
            if (!string.IsNullOrEmpty(o.level)) {
                prefix = usInfo.ToTitleCase(usInfo.ToLower(o.level));
            } else {
                prefix = "*** o.level was null or empty in MessageProgress bw_ProgressChanged ****";
            }
            string msg = string.Format("{0} - {1} - {2}", Messenger.timeNow(), prefix, o.message);

            if (o.progress != null) {
                this.pw.ProgressIncrement((int) o.progress, msg);
            } else if (o.adjustMaxBy != null) {
                this.pw.ProgressAdjustMaximum((int) o.adjustMaxBy);
            } else if (o.resetTo != null) {
                this.pw.ProgressReset((int) o.resetTo);
            } else if (o.message != null) {
                switch (o.level) {
                    case "DEBUG":
                        this.pw.sendDebug(msg);
                        break;
                    case "INFO":
                        this.pw.sendInfo(msg);
                        break;
                    case "WARNING":
                        this.pw.sendWarning(msg);
                        break;
                    case "ERROR":
                        this.pw.sendError(msg);
                        break;
                    default:
                        this.pw.sendDebug(msg);
                        break;
                }
            } 
        }

        public override void update(updateObject o) {
            this.bw.ReportProgress(0, o);
            // altering control from this thread locks UI this.m.sendError(message);,this.m.ProgressIncrement(progress, message);
        }

        public override void sendDebug(string message) {
        }

        public override void sendInfo(string message) {
            if (this.logLevel >= OdctSettings.LOGLEVEL_INFO) {
                this.update(new updateObject() { message = message, level = "INFO" });
            }
        }

        public override void sendWarning(string message) {
            if (this.logLevel >= OdctSettings.LOGLEVEL_WARNING) {
                this.update(new updateObject() { message = message, level = "WARNING" });
            }
        }

        public override void sendError(string message) {
            if (this.logLevel >= OdctSettings.LOGLEVEL_ERROR) {
                this.update(new updateObject() { message = message, level = "ERROR" });
            }
        }

    }
}
