﻿using System;
using System.ComponentModel;
using System.Windows.Forms;
using System.Threading;
using odct;

namespace odct.View {
    public partial class ProgressWindow : Form {
        Action<object> cancel;

        public ProgressWindow(Action<object> cancel) {
            this.cancel = cancel;
            this.InitializeComponent();
            this.FormClosing += ProgressWindow_FormClosing;
        }

        public void ProgressReset(int newMax) {
            this.ProgressBarControl.Minimum = 0;
            this.ProgressBarControl.Maximum = newMax;
            this.ProgressBarControl.Step = 1;
            this.ProgressBarControl.Value = 0;
            this.ProgressBarControl.Visible = true;
        }

        public void ProgressIncrement(int increments, string message = null) {
            this.appendTextArea(message);
            for (int i = 0; i < increments; i++) {
                this.ProgressBarControl.PerformStep();
            }
        }

        public void ProgressAdjustMaximum(int changeToMaxWork) {
            this.ProgressBarControl.Maximum += changeToMaxWork;
        }

        private void CancelButton_Click(object sender, EventArgs e) {
        }

        private void ProgressWindow_FormClosing(object sender, FormClosingEventArgs e) {
            if (!"E&xit".Equals(this.CancelProgressButton.Text)) {
                var result = MessageBox.Show(
                    "Exiting or Canceling will terminate the data collection.\nAre you sure you want to exit?",
                    "Data Collection Halt",
                    MessageBoxButtons.YesNo,
                    MessageBoxIcon.Question);
                if (result == DialogResult.No) {
                    e.Cancel = true; // ie do cancel the close event
                } else {
                    this.cancel(e); // kill all of the collector threads
                }
            }

        }

        private void appendTextArea(string message) {
            this.LogMessagesTextBox.AppendText(string.Format("{0}\n", message));
            this.LogMessagesTextBox.ClearUndo();
        }

        public void sendDebug(string message) {
        }

        public void sendInfo(string message) {
            this.appendTextArea(message);
        }

        public void sendWarning(string message) {
            this.appendTextArea(message);
        }

        public void sendError(string message) {
            this.appendTextArea(message);
        }

        public void SetProgressLabelStep(string message) {
            this.ProgressLabel.Text = message;
        }

        public void SetButtonLabel(string message) {
            this.CancelProgressButton.Text = message;
        }

        public void closeMessenger() {
            this.appendTextArea("DONE");
        }
    }
}
